@@ -26,7 +26,9 @@ import java.util.Date;
 import java.util.HashMap;
 
 import org.eclipse.swt.SWT;
+import org.eclipse.swt.widgets.Event;
 import org.eclipse.swt.widgets.FileDialog;
+import org.eclipse.swt.widgets.Listener;
 import org.supercsv.io.CsvMapWriter;
 import org.supercsv.io.ICsvMapWriter;
 import org.supercsv.prefs.CsvPreference;
@@ -35,7 +37,9 @@ import com.itextpdf.text.BaseColor;
 import com.itextpdf.text.Element;
 
 import de.jost_net.JVerein.Einstellungen;
+import de.jost_net.JVerein.gui.action.ArbeitseinsatzAction;
 import de.jost_net.JVerein.gui.input.ArbeitseinsatzUeberpruefungInput;
+import de.jost_net.JVerein.gui.menu.ArbeitseinsatzMenu;
 import de.jost_net.JVerein.gui.parts.ArbeitseinsatzPart;
 import de.jost_net.JVerein.gui.parts.ArbeitseinsatzUeberpruefungList;
 import de.jost_net.JVerein.io.ArbeitseinsatzZeile;
@@ -46,17 +50,21 @@ import de.jost_net.JVerein.rmi.Arbeitseinsatz;
 import de.jost_net.JVerein.rmi.Mitglied;
 import de.jost_net.JVerein.rmi.Zusatzbetrag;
 import de.jost_net.JVerein.util.Dateiname;
+import de.jost_net.JVerein.util.JVDateFormatTTMMJJJJ;
 import de.willuhn.datasource.GenericIterator;
 import de.willuhn.datasource.GenericObject;
 import de.willuhn.datasource.pseudo.PseudoIterator;
 import de.willuhn.datasource.rmi.DBIterator;
-import de.willuhn.jameica.gui.AbstractControl;
+import de.willuhn.datasource.rmi.DBService;
 import de.willuhn.jameica.gui.AbstractView;
 import de.willuhn.jameica.gui.Action;
 import de.willuhn.jameica.gui.GUI;
 import de.willuhn.jameica.gui.Part;
+import de.willuhn.jameica.gui.formatter.CurrencyFormatter;
+import de.willuhn.jameica.gui.formatter.DateFormatter;
 import de.willuhn.jameica.gui.input.SelectInput;
 import de.willuhn.jameica.gui.parts.Button;
+import de.willuhn.jameica.gui.parts.TablePart;
 import de.willuhn.jameica.system.Application;
 import de.willuhn.jameica.system.BackgroundTask;
 import de.willuhn.jameica.system.Settings;
@@ -64,12 +72,10 @@ import de.willuhn.logging.Logger;
 import de.willuhn.util.ApplicationException;
 import de.willuhn.util.ProgressMonitor;
 
-public class ArbeitseinsatzControl extends AbstractControl
+public class ArbeitseinsatzControl extends FilterControl
 {
   private ArbeitseinsatzPart part = null;
 
-  private Settings settings = null;
-
   private Arbeitseinsatz aeins = null;
 
   private ArbeitseinsatzUeberpruefungList arbeitseinsatzueberpruefungList;
@@ -77,6 +83,8 @@ public class ArbeitseinsatzControl extends AbstractControl
   private SelectInput suchjahr = null;
 
   private ArbeitseinsatzUeberpruefungInput auswertungschluessel = null;
+  
+  private TablePart arbeitseinsatzList;
 
   public ArbeitseinsatzControl(AbstractView view)
   {
@@ -101,7 +109,7 @@ public class ArbeitseinsatzControl extends AbstractControl
     {
       return part;
     }
-    part = new ArbeitseinsatzPart(getArbeitseinsatz());
+    part = new ArbeitseinsatzPart(getArbeitseinsatz(), true);
     return part;
   }
 
@@ -113,6 +121,7 @@ public class ArbeitseinsatzControl extends AbstractControl
       return auswertungschluessel;
     }
     auswertungschluessel = new ArbeitseinsatzUeberpruefungInput(1);
+    auswertungschluessel.addListener(new FilterListener());
     return auswertungschluessel;
   }
 
@@ -121,6 +130,18 @@ public class ArbeitseinsatzControl extends AbstractControl
     try
     {
       Arbeitseinsatz ae = getArbeitseinsatz();
+      if (ae.isNewObject())
+      {
+        if (getPart().getMitglied().getValue() != null)
+        {
+          Mitglied m = (Mitglied) getPart().getMitglied().getValue();
+          ae.setMitglied(Integer.parseInt(m.getID()));
+        }
+        else
+        {
+          throw new ApplicationException("Bitte Mitglied eingeben");
+        }
+      }
       ae.setDatum((Date) part.getDatum().getValue());
       ae.setStunden((Double) part.getStunden().getValue());
       ae.setBemerkung((String) part.getBemerkung().getValue());
@@ -166,12 +187,13 @@ public class ArbeitseinsatzControl extends AbstractControl
     suchjahr = new SelectInput(jahre, settings.getInt("jahr", jahre.get(0)));
     // suchjahr.setPleaseChoose("Bitte auswhlen");
     suchjahr.setPreselected(settings.getInt("jahr", bis.get(Calendar.YEAR)));
+    suchjahr.addListener(new FilterListener());
     return suchjahr;
   }
 
   public Button getPDFAusgabeButton()
   {
-    Button b = new Button("PDF-Ausgabe", new Action()
+    Button b = new Button("PDF", new Action()
     {
 
       @Override
@@ -188,13 +210,13 @@ public class ArbeitseinsatzControl extends AbstractControl
               "Fehler beim Start der PDF-Ausgabe der Arbeitseinsatzberprfung");
         }
       }
-    }, null, true, "file-pdf.png");
+    }, null, false, "file-pdf.png");
     return b;
   }
 
   public Button getCSVAusgabeButton()
   {
-    Button b = new Button("CSV-Ausgabe", new Action()
+    Button b = new Button("CSV", new Action()
     {
 
       @Override
@@ -211,7 +233,7 @@ public class ArbeitseinsatzControl extends AbstractControl
               "Fehler beim Start der CSV-Ausgabe der Arbeitseinsatzberprfung");
         }
       }
-    }, null, true, "code.png");
+    }, null, false, "xsd.png");
     return b;
   }
 
@@ -234,7 +256,7 @@ public class ArbeitseinsatzControl extends AbstractControl
               "Fehler beim der Zusatzbetragsgenerierung");
         }
       }
-    }, null, true, "euro-sign.png");
+    }, null, false, "euro-sign.png");
     return b;
   }
 
@@ -249,17 +271,17 @@ public class ArbeitseinsatzControl extends AbstractControl
       fd.setFilterPath(path);
     }
     fd.setFileName(new Dateiname("arbeitseinsaetze", "",
-        Einstellungen.getEinstellung().getDateinamenmuster(), "PDF").get());
-    fd.setFilterExtensions(new String[] { "*.PDF" });
+        Einstellungen.getEinstellung().getDateinamenmuster(), "pdf").get());
+    fd.setFilterExtensions(new String[] { "*.pdf" });
 
     String s = fd.open();
     if (s == null || s.length() == 0)
     {
       return;
     }
-    if (!s.endsWith(".PDF"))
+    if (!s.toLowerCase().endsWith(".pdf"))
     {
-      s = s + ".PDF";
+      s = s + ".pdf";
     }
     final File file = new File(s);
     final GenericIterator<ArbeitseinsatzZeile> it = getIterator();
@@ -342,17 +364,17 @@ public class ArbeitseinsatzControl extends AbstractControl
       fd.setFilterPath(path);
     }
     fd.setFileName(new Dateiname("arbeitseinsaetze", "",
-        Einstellungen.getEinstellung().getDateinamenmuster(), "CSV").get());
-    fd.setFilterExtensions(new String[] { "*.CSV" });
+        Einstellungen.getEinstellung().getDateinamenmuster(), "csv").get());
+    fd.setFilterExtensions(new String[] { "*.csv" });
 
     String s = fd.open();
     if (s == null || s.length() == 0)
     {
       return;
     }
-    if (!s.endsWith(".CSV"))
+    if (!s.toLowerCase().endsWith(".csv"))
     {
-      s = s + ".CSV";
+      s = s + ".csv";
     }
     final File file = new File(s);
     final GenericIterator<ArbeitseinsatzZeile> it = getIterator();
@@ -451,7 +473,7 @@ public class ArbeitseinsatzControl extends AbstractControl
             zb.setFaelligkeit(new Date());
             zb.setStartdatum(new Date());
             zb.setIntervall(IntervallZusatzzahlung.KEIN);
-            zb.setMitglied(new Integer((String) z.getAttribute("mitgliedid")));
+            zb.setMitglied(Integer.valueOf((String) z.getAttribute("mitgliedid")));
             zb.store();
           }
           GUI.getStatusBar().setSuccessText("Liste Arbeitseinstze gestartet");
@@ -487,7 +509,7 @@ public class ArbeitseinsatzControl extends AbstractControl
   {
     ArrayList<ArbeitseinsatzZeile> zeile = arbeitseinsatzueberpruefungList
         .getInfo();
-
+    @SuppressWarnings("unchecked")
     GenericIterator<ArbeitseinsatzZeile> gi = PseudoIterator
         .fromArray(zeile.toArray(new GenericObject[zeile.size()]));
     return gi;
@@ -529,4 +551,117 @@ public class ArbeitseinsatzControl extends AbstractControl
     }
     return arbeitseinsatzueberpruefungList.getArbeitseinsatzUeberpruefungList();
   }
+  
+  private void refreshList()
+  {
+    try
+    {
+      getArbeitseinsatzUeberpruefungList();
+    }
+    catch (ApplicationException e1)
+    {
+      //
+    }
+  }
+  
+  private class FilterListener implements Listener
+  {
+
+    @Override
+    public void handleEvent(Event event)
+    {
+      if (event.type != SWT.Selection && event.type != SWT.FocusOut)
+      {
+        return;
+      }
+      refreshList();
+    }
+  }
+  
+  public Part getArbeitseinsatzTable() throws RemoteException
+  {
+    if (arbeitseinsatzList != null)
+    {
+      return arbeitseinsatzList;
+    }
+    
+    DBIterator<Arbeitseinsatz> arbeitseinsaetze = getArbeitseinsaetzeIt();
+    arbeitseinsatzList = new TablePart(arbeitseinsaetze,
+        new ArbeitseinsatzAction(null));
+    arbeitseinsatzList.setRememberColWidths(true);
+    arbeitseinsatzList.setRememberOrder(true);
+    arbeitseinsatzList.setContextMenu(new ArbeitseinsatzMenu());
+    arbeitseinsatzList.addColumn("Name", "mitglied");
+    arbeitseinsatzList.addColumn("Datum", "datum",
+        new DateFormatter(new JVDateFormatTTMMJJJJ()));
+    arbeitseinsatzList.addColumn("Stunden", "stunden",
+        new CurrencyFormatter("", Einstellungen.DECIMALFORMAT));
+    arbeitseinsatzList.addColumn("Bemerkung", "bemerkung");
+    return arbeitseinsatzList;
+  }
+  
+  
+  public void TabRefresh()
+  {
+    try
+    {
+      if (arbeitseinsatzList == null)
+      {
+        return;
+      }
+      arbeitseinsatzList.removeAll();
+      DBIterator<Arbeitseinsatz> arbeitseinsaetze = getArbeitseinsaetzeIt();
+      while (arbeitseinsaetze.hasNext())
+      {
+        arbeitseinsatzList.addItem(arbeitseinsaetze.next());
+      }
+    }
+    catch (RemoteException e1)
+    {
+      Logger.error("Fehler", e1);
+    }
+  }
+  
+  private DBIterator<Arbeitseinsatz> getArbeitseinsaetzeIt() throws RemoteException
+  {
+    DBService service = Einstellungen.getDBService();
+    DBIterator<Arbeitseinsatz> arbeitseinsaetze = service
+        .createList(Arbeitseinsatz.class);
+    arbeitseinsaetze.join("mitglied");
+    arbeitseinsaetze.addFilter("mitglied.id = arbeitseinsatz.mitglied");
+    
+    if (isSuchnameAktiv() && getSuchname().getValue() != null)
+    {
+      String tmpSuchname = (String) getSuchname().getValue();
+      if (tmpSuchname.length() > 0)
+      {
+        String suchName = "%" + tmpSuchname.toLowerCase() + "%";
+        arbeitseinsaetze.addFilter("(lower(name) like ? "
+            + "or lower(vorname) like ?)" , 
+            new Object[] { suchName, suchName });
+      }
+    }
+    if (isDatumvonAktiv() && getDatumvon().getValue() != null)
+    {
+      arbeitseinsaetze.addFilter("datum >= ?",
+          new Object[] { (Date) getDatumvon().getValue() });
+    }
+    if (isDatumbisAktiv() && getDatumbis().getValue() != null)
+    {
+      arbeitseinsaetze.addFilter("datum <= ?",
+          new Object[] { (Date) getDatumbis().getValue() });
+    }
+    if (isSuchtextAktiv() && getSuchtext().getValue() != null)
+    {
+      String tmpSuchtext = (String) getSuchtext().getValue();
+      if (tmpSuchtext.length() > 0)
+      {
+        arbeitseinsaetze.addFilter("(lower(bemerkung) like ?)",
+            new Object[] { "%" + tmpSuchtext.toLowerCase() + "%"});
+      }
+    }
+    arbeitseinsaetze.setOrder("ORDER by datum desc");
+    return arbeitseinsaetze;
+  }
+  
 }
