@@ -19,7 +19,6 @@ package de.jost_net.JVerein.gui.control;
 import java.io.File;
 import java.io.FileOutputStream;
 import java.rmi.RemoteException;
-import java.text.ParseException;
 import java.util.Date;
 
 import org.eclipse.swt.SWT;
@@ -44,7 +43,6 @@ import de.jost_net.JVerein.rmi.Kursteilnehmer;
 import de.jost_net.JVerein.util.Dateiname;
 import de.jost_net.JVerein.util.JVDateFormatTTMMJJJJ;
 import de.willuhn.datasource.rmi.DBIterator;
-import de.willuhn.jameica.gui.AbstractControl;
 import de.willuhn.jameica.gui.AbstractView;
 import de.willuhn.jameica.gui.Action;
 import de.willuhn.jameica.gui.GUI;
@@ -57,6 +55,8 @@ import de.willuhn.jameica.gui.input.Input;
 import de.willuhn.jameica.gui.input.TextInput;
 import de.willuhn.jameica.gui.parts.Button;
 import de.willuhn.jameica.gui.parts.TablePart;
+import de.willuhn.jameica.hbci.HBCIProperties;
+import de.willuhn.jameica.hbci.gui.formatter.IbanFormatter;
 import de.willuhn.jameica.system.Application;
 import de.willuhn.jameica.system.BackgroundTask;
 import de.willuhn.jameica.system.Settings;
@@ -64,7 +64,7 @@ import de.willuhn.logging.Logger;
 import de.willuhn.util.ApplicationException;
 import de.willuhn.util.ProgressMonitor;
 
-public class KursteilnehmerControl extends AbstractControl
+public class KursteilnehmerControl extends FilterControl
 {
 
   private PersonenartInput personenart;
@@ -107,19 +107,6 @@ public class KursteilnehmerControl extends AbstractControl
 
   private TablePart part;
 
-  // Elemente fr die Auswertung
-
-  private DateInput eingabedatumvon = null;
-
-  private DateInput eingabedatumbis = null;
-
-  private DateInput abbuchungsdatumvon = null;
-
-  private DateInput abbuchungsdatumbis = null;
-
-  private TextInput suchname = null;
-
-  private Settings settings = null;
 
   public KursteilnehmerControl(AbstractView view)
   {
@@ -152,7 +139,7 @@ public class KursteilnehmerControl extends AbstractControl
       public void handleEvent(Event event)
       {
         String pa = (String) personenart.getValue();
-        if (pa.startsWith("n"))
+        if (pa.toLowerCase().startsWith("n"))
         {
           name.setName("Name");
           vorname.setName("Vorname");
@@ -308,7 +295,8 @@ public class KursteilnehmerControl extends AbstractControl
     this.mandatdatum.setTitle("Datum des Mandats");
     this.mandatdatum.setName("Datum des Mandats");
     this.mandatdatum.setText("Bitte Datum des Mandats whlen");
-    mandatdatum.setName("Datum des Mandats");
+    this.mandatdatum.setName("Datum des Mandats");
+    this.mandatdatum.setMandatory(true);
     return mandatdatum;
   }
 
@@ -330,7 +318,7 @@ public class KursteilnehmerControl extends AbstractControl
     {
       return iban;
     }
-    iban = new IBANInput(getKursteilnehmer().getIban(), getBIC());
+    iban = new IBANInput(HBCIProperties.formatIban(getKursteilnehmer().getIban()), getBIC());
     iban.setName("IBAN");
     iban.setMandatory(true);
     return iban;
@@ -358,8 +346,11 @@ public class KursteilnehmerControl extends AbstractControl
     Date d = getKursteilnehmer().getGeburtsdatum();
     this.geburtsdatum = new DateInput(d, new JVDateFormatTTMMJJJJ());
     this.geburtsdatum.setTitle("Geburtsdatum");
-    this.geburtsdatum.setText(("Bitte Geburtsdatum whlen"));
-    this.geburtsdatum.setMandatory(true);
+    this.geburtsdatum.setText("Bitte Geburtsdatum whlen");
+    if (Einstellungen.getEinstellung().getKursteilnehmerGebGesPflicht())
+    {
+      this.geburtsdatum.setMandatory(true);
+    }
     return geburtsdatum;
   }
 
@@ -372,128 +363,19 @@ public class KursteilnehmerControl extends AbstractControl
     geschlecht = new GeschlechtInput(getKursteilnehmer().getGeschlecht());
     geschlecht.setName("Geschlecht");
     geschlecht.setPleaseChoose("Bitte auswhlen");
-    geschlecht.setMandatory(true);
+    if (Einstellungen.getEinstellung().getKursteilnehmerGebGesPflicht())
+    {
+      geschlecht.setMandatory(true);
+    }
     return geschlecht;
   }
 
-  public TextInput getSuchname()
-  {
-    if (suchname != null)
-    {
-      return suchname;
-    }
-    String tmp = settings.getString("name", "");
-    this.suchname = new TextInput(tmp, 30);
-    suchname.addListener(new FilterListener());
-    return suchname;
-  }
-
-  public DateInput getEingabedatumvon()
-  {
-    if (eingabedatumvon != null)
-    {
-      return eingabedatumvon;
-    }
-    Date d = null;
-    String tmp = settings.getString("eingabedatum.von", null);
-    if (tmp != null)
-    {
-      try
-      {
-        d = new JVDateFormatTTMMJJJJ().parse(tmp);
-      }
-      catch (ParseException e)
-      {
-        //
-      }
-    }
-    this.eingabedatumvon = new DateInput(d, new JVDateFormatTTMMJJJJ());
-    this.eingabedatumvon.setTitle("Eingabedatum");
-    this.eingabedatumvon.setText("Beginn des Eingabe-Zeitraumes");
-    eingabedatumvon.addListener(new FilterListener());
-    return eingabedatumvon;
-  }
-
-  public DateInput getEingabedatumbis()
-  {
-    if (eingabedatumbis != null)
-    {
-      return eingabedatumbis;
-    }
-    Date d = null;
-    String tmp = settings.getString("eingabedatum.bis", null);
-    if (tmp != null)
-    {
-      try
-      {
-        d = new JVDateFormatTTMMJJJJ().parse(tmp);
-      }
-      catch (ParseException e)
-      {
-        //
-      }
-    }
-    this.eingabedatumbis = new DateInput(d, new JVDateFormatTTMMJJJJ());
-    this.eingabedatumbis.setTitle("Eingabedatum");
-    this.eingabedatumbis.setText("Ende des Eingabe-Zeitraumes");
-    eingabedatumbis.addListener(new FilterListener());
-    return eingabedatumbis;
-  }
-
-  public DateInput getAbbuchungsdatumvon()
-  {
-    if (abbuchungsdatumvon != null)
-    {
-      return abbuchungsdatumvon;
-    }
-    Date d = null;
-    String tmp = settings.getString("abbuchungsdatum.von", null);
-    if (tmp != null)
-    {
-      try
-      {
-        d = new JVDateFormatTTMMJJJJ().parse(tmp);
-      }
-      catch (ParseException e)
-      {
-        //
-      }
-    }
-    this.abbuchungsdatumvon = new DateInput(d, new JVDateFormatTTMMJJJJ());
-    this.abbuchungsdatumvon.setTitle("Abbuchungsdatum");
-    this.abbuchungsdatumvon.setText("Beginn des Abbuchungszeitraumes");
-    return abbuchungsdatumvon;
-  }
-
-  public DateInput getAbbuchungsdatumbis()
-  {
-    if (abbuchungsdatumbis != null)
-    {
-      return abbuchungsdatumbis;
-    }
-    Date d = null;
-    String tmp = settings.getString("abbuchungsdatum.bis", null);
-    if (tmp != null)
-    {
-      try
-      {
-        d = new JVDateFormatTTMMJJJJ().parse(tmp);
-      }
-      catch (ParseException e)
-      {
-        //
-      }
-    }
-    this.abbuchungsdatumbis = new DateInput(d, new JVDateFormatTTMMJJJJ());
-    this.abbuchungsdatumbis.setTitle("Abbuchungsdatum");
-    this.abbuchungsdatumbis.setText("Ende des Abbuchungszeitraumes");
-    return abbuchungsdatumbis;
-  }
-
   public Part getKursteilnehmerTable() throws RemoteException
   {
-    saveDefaults();
-
+    if (part != null)
+    {
+      return part;
+    }
     DBIterator<Kursteilnehmer> kursteilnehmer = getIterator();
     part = new TablePart(kursteilnehmer, new KursteilnehmerDetailAction());
 
@@ -504,7 +386,7 @@ public class KursteilnehmerControl extends AbstractControl
     part.addColumn("Ort", "ort");
     part.addColumn("Verwendungszweck", "vzweck1");
     part.addColumn("BIC", "bic");
-    part.addColumn("IBAN", "iban");
+    part.addColumn("IBAN", "iban", new IbanFormatter());
     part.addColumn("Betrag", "betrag",
         new CurrencyFormatter("", Einstellungen.DECIMALFORMAT));
     part.addColumn("Mandats-ID", "mandatid");
@@ -513,16 +395,16 @@ public class KursteilnehmerControl extends AbstractControl
     part.addColumn("Abbuchungsdatum", "abbudatum",
         new DateFormatter(new JVDateFormatTTMMJJJJ()));
     part.setContextMenu(new KursteilnehmerMenu(part));
+    part.setMulti(true);
 
     return part;
   }
 
-  private void refresh()
+  public void TabRefresh()
   {
 
     try
     {
-      saveDefaults();
       if (part == null)
       {
         return;
@@ -543,7 +425,7 @@ public class KursteilnehmerControl extends AbstractControl
 
   public Button getStartAuswertungButton()
   {
-    Button b = new Button("starten", new Action()
+    Button b = new Button("Starten", new Action()
     {
 
       @Override
@@ -556,76 +438,6 @@ public class KursteilnehmerControl extends AbstractControl
     return b;
   }
 
-  /**
-   * Default-Werte speichern.
-   * 
-   * @throws RemoteException
-   */
-  public void saveDefaults()
-  {
-    if (this.suchname != null)
-    {
-      settings.setAttribute("name", (String) getSuchname().getValue());
-    }
-
-    if (this.eingabedatumvon != null)
-    {
-      Date tmp = (Date) getEingabedatumvon().getValue();
-      if (tmp != null)
-      {
-        settings.setAttribute("eingabedatum.von",
-            new JVDateFormatTTMMJJJJ().format(tmp));
-      }
-      else
-      {
-        settings.setAttribute("eingabedatum.von", "");
-      }
-    }
-
-    if (this.eingabedatumbis != null)
-    {
-      Date tmp = (Date) getEingabedatumbis().getValue();
-      if (tmp != null)
-      {
-        settings.setAttribute("eingabedatum.bis",
-            new JVDateFormatTTMMJJJJ().format(tmp));
-      }
-      else
-      {
-        settings.setAttribute("eingabedatum.bis", "");
-      }
-    }
-
-    if (this.abbuchungsdatumvon != null)
-    {
-      Date tmp = (Date) getAbbuchungsdatumvon().getValue();
-      if (tmp != null)
-      {
-        settings.setAttribute("abbuchungsdatum.von",
-            new JVDateFormatTTMMJJJJ().format(tmp));
-      }
-      else
-      {
-        settings.setAttribute("abbuchungsdatum.von", "");
-      }
-    }
-
-    if (this.abbuchungsdatumbis != null)
-    {
-      Date tmp = (Date) getAbbuchungsdatumbis().getValue();
-      if (tmp != null)
-      {
-        settings.setAttribute("abbuchungsdatum.bis",
-            new JVDateFormatTTMMJJJJ().format(tmp));
-      }
-      else
-      {
-        settings.setAttribute("abbuchungsdatum.bis", "");
-      }
-    }
-
-  }
-
   public void handleStore()
   {
     try
@@ -646,10 +458,14 @@ public class KursteilnehmerControl extends AbstractControl
       k.setEmail((String) getEmail().getValue());
       k.setVZweck1((String) getVZweck1().getValue());
       k.setMandatDatum((Date) getMandatDatum().getValue());
-      k.setIban((String) getIBAN().getValue());
+      String ib = (String) getIBAN().getValue();
+      if (ib == null)
+        k.setIban(null);
+      else
+        k.setIban(ib.toUpperCase().replace(" ",""));
       k.setBic((String) getBIC().getValue());
       k.setBetrag((Double) getBetrag().getValue());
-      if (Einstellungen.getEinstellung().getGeburtsdatumPflicht())
+      if (Einstellungen.getEinstellung().getKursteilnehmerGebGesPflicht())
       {
         k.setGeburtsdatum((Date) getGeburtsdatum().getValue());
         k.setGeschlecht((String) getGeschlecht().getValue());
@@ -676,33 +492,16 @@ public class KursteilnehmerControl extends AbstractControl
   private void starteAuswertung()
   {
     // Alle Kursteilnehmer lesen
-    final DBIterator<Kursteilnehmer> list;
+    
     try
     {
-      saveDefaults();
+      saveFilterSettings();
       String subtitle = "";
-      list = Einstellungen.getDBService().createList(Kursteilnehmer.class);
-      if (abbuchungsdatumvon.getValue() != null)
-      {
-        Date d = (Date) abbuchungsdatumvon.getValue();
-        subtitle += "Abbuchungsdatum von" + " "
-            + new JVDateFormatTTMMJJJJ().format(d) + "  ";
-        list.addFilter("abbudatum >= ?",
-            new Object[] { new java.sql.Date(d.getTime()) });
-      }
-      if (abbuchungsdatumbis.getValue() != null)
-      {
-        Date d = (Date) abbuchungsdatumbis.getValue();
-        subtitle += " " + "bis" + " " + new JVDateFormatTTMMJJJJ().format(d)
-            + "  ";
-        list.addFilter("abbudatum <= ?",
-            new Object[] { new java.sql.Date(d.getTime()) });
-      }
+      final DBIterator<Kursteilnehmer> list = getIterator();
+      
       FileDialog fd = new FileDialog(GUI.getShell(), SWT.SAVE);
       fd.setText("Ausgabedatei whlen.");
 
-      Settings settings = new Settings(this.getClass());
-
       String path = settings.getString("lastdir",
           System.getProperty("user.home"));
       if (path != null && path.length() > 0)
@@ -792,40 +591,48 @@ public class KursteilnehmerControl extends AbstractControl
     }
   }
 
-  private class FilterListener implements Listener
-  {
-
-    @Override
-    public void handleEvent(Event event)
-    {
-      if (event.type != SWT.Selection && event.type != SWT.FocusOut)
-      {
-        return;
-      }
-      refresh();
-    }
-  }
-
   private DBIterator<Kursteilnehmer> getIterator() throws RemoteException
   {
     DBIterator<Kursteilnehmer> kursteilnehmer = Einstellungen.getDBService()
         .createList(Kursteilnehmer.class);
-    String suchN = (String) getSuchname().getValue();
-    if (suchN != null && suchN.length() > 0)
+    if (isSuchnameAktiv() && getSuchname().getValue() != null)
     {
-      kursteilnehmer.addFilter("name like ?",
-          new Object[] { "%" + suchN + "%" });
+      String suchN = (String) getSuchname().getValue();
+      if (suchN.length() > 0)
+      {
+        kursteilnehmer.addFilter("name like ?",
+            new Object[] { "%" + suchN + "%" });
+      }
     }
-    if (getEingabedatumvon().getValue() != null)
+    if (isSuchtextAktiv() && getSuchtext().getValue() != null)
+    {
+      String tmpSuchtext = (String) getSuchtext().getValue();
+      if (tmpSuchtext.length() > 0)
+      {
+        kursteilnehmer.addFilter("(lower(vzweck1) like ?)",
+            new Object[] { "%" + tmpSuchtext.toLowerCase() + "%"});
+      }
+    }
+    if (isEingabedatumvonAktiv() && getEingabedatumvon().getValue() != null)
     {
       kursteilnehmer.addFilter("eingabedatum >= ?",
           new Object[] { (Date) getEingabedatumvon().getValue() });
     }
-    if (getEingabedatumbis().getValue() != null)
+    if (isEingabedatumbisAktiv() && getEingabedatumbis().getValue() != null)
     {
       kursteilnehmer.addFilter("eingabedatum <= ?",
           new Object[] { (Date) getEingabedatumbis().getValue() });
     }
+    if (isAbbuchungsdatumvonAktiv() && getAbbuchungsdatumvon().getValue() != null)
+    {
+      kursteilnehmer.addFilter("abbudatum >= ?",
+          new Object[] { (Date) getAbbuchungsdatumvon().getValue() });
+    }
+    if (isAbbuchungsdatumbisAktiv() && getAbbuchungsdatumbis().getValue() != null)
+    {
+      kursteilnehmer.addFilter("abbudatum <= ?",
+          new Object[] { (Date) getAbbuchungsdatumbis().getValue() });
+    }
     return kursteilnehmer;
   }
 
