@@ -21,6 +21,7 @@ import java.io.IOException;
 import java.io.StringWriter;
 import java.rmi.RemoteException;
 import java.sql.Timestamp;
+import java.util.ArrayList;
 import java.util.Date;
 import java.util.Map;
 import java.util.TreeSet;
@@ -36,11 +37,10 @@ import de.jost_net.JVerein.Einstellungen;
 import de.jost_net.JVerein.Variable.AllgemeineMap;
 import de.jost_net.JVerein.Variable.LastschriftMap;
 import de.jost_net.JVerein.Variable.VarTools;
-import de.jost_net.JVerein.gui.input.FormularInput;
 import de.jost_net.JVerein.io.Ct1Ueberweisung;
 import de.jost_net.JVerein.io.FormularAufbereitung;
 import de.jost_net.JVerein.io.MailSender;
-import de.jost_net.JVerein.keys.Abrechnungsausgabe;
+import de.jost_net.JVerein.keys.Ct1Ausgabe;
 import de.jost_net.JVerein.keys.FormularArt;
 import de.jost_net.JVerein.rmi.Abrechnungslauf;
 import de.jost_net.JVerein.rmi.Formular;
@@ -53,53 +53,25 @@ import de.jost_net.JVerein.util.Datum;
 import de.jost_net.JVerein.util.JVDateFormatDATETIME;
 import de.jost_net.JVerein.util.JVDateFormatTTMMJJJJ;
 import de.willuhn.datasource.rmi.DBIterator;
-import de.willuhn.jameica.gui.AbstractControl;
 import de.willuhn.jameica.gui.AbstractView;
 import de.willuhn.jameica.gui.Action;
 import de.willuhn.jameica.gui.GUI;
 import de.willuhn.jameica.gui.input.DateInput;
 import de.willuhn.jameica.gui.input.SelectInput;
-import de.willuhn.jameica.gui.input.TextAreaInput;
 import de.willuhn.jameica.gui.input.TextInput;
 import de.willuhn.jameica.gui.parts.Button;
 import de.willuhn.jameica.system.Application;
 import de.willuhn.jameica.system.BackgroundTask;
+import de.willuhn.jameica.system.OperationCanceledException;
 import de.willuhn.jameica.system.Settings;
 import de.willuhn.logging.Logger;
-import de.willuhn.util.ApplicationException;
 import de.willuhn.util.ProgressMonitor;
 
-public class PreNotificationControl extends AbstractControl
+public class PreNotificationControl extends DruckMailControl
 {
 
-  private Settings settings = null;
-
   private TabFolder folder = null;
 
-  private SelectInput output = null;
-
-  private TextInput mailsubject = null;
-
-  private TextAreaInput mailbody = null;
-
-  public static final String EMAIL = "EMail";
-
-  public static final String PDF1 = "PDF (Lastschriften ohne Mailadresse)";
-
-  public static final String PDF2 = "PDF (Alle)";
-
-  private SelectInput pdfModus = null;
-
-  public static final String NICHT_EINZELN = "eine PDF-Datei";
-
-  public static final String EINZELN_NUMMERIERT = "einzelne PDF-Dateien, nummeriert";
-
-  public static final String EINZELN_MITGLIEDSNUMMER = "einzelne PDF-Dateien, mit Mitgliedsnummer";
-
-  public static final String EINZELN_NUMMERIERT_UND_MNR = "einzelne PDF-Dateien, nummeriert mit Mitgliedsnummer";
-
-  private FormularInput formular = null;
-
   private FormularAufbereitung fa;
 
   private DateInput ausfuehrungsdatum;
@@ -122,71 +94,10 @@ public class PreNotificationControl extends AbstractControl
       return folder;
     }
     folder = new TabFolder(parent, SWT.NONE);
-    folder.setSelection(settings.getInt("tab.selection", 0));
+    folder.setSelection(settings.getInt(settingsprefix + "tab.selection", 0));
     return folder;
   }
 
-  public SelectInput getPdfModus()
-  {
-    if (pdfModus != null)
-    {
-      return pdfModus;
-    }
-    Object[] values = new Object[] { NICHT_EINZELN, EINZELN_NUMMERIERT,
-        EINZELN_MITGLIEDSNUMMER, EINZELN_NUMMERIERT_UND_MNR };
-    pdfModus = new SelectInput(values,
-        settings.getString("pdfModus", NICHT_EINZELN));
-    pdfModus.setName("PDF als");
-    return pdfModus;
-  }
-
-  public SelectInput getOutput()
-  {
-    if (output != null)
-    {
-      return output;
-    }
-    Object[] values = new Object[] { EMAIL, PDF1, PDF2 };
-    output = new SelectInput(values, settings.getString("output", PDF1));
-    output.setName("Ausgabe");
-    return output;
-  }
-
-  public FormularInput getFormular(FormularArt formulartyp)
-      throws RemoteException
-  {
-    if (formular != null)
-    {
-      return formular;
-    }
-    formular = new FormularInput(formulartyp);
-    return formular;
-  }
-
-  public TextInput getMailSubject()
-  {
-    if (mailsubject != null)
-    {
-      return mailsubject;
-    }
-    mailsubject = new TextInput(settings.getString("mail.subject", ""), 100);
-    mailsubject.setName("Betreff");
-    return mailsubject;
-
-  }
-
-  public TextAreaInput getMailBody() throws RemoteException
-  {
-    if (mailbody != null)
-    {
-      return mailbody;
-    }
-    mailbody = new TextAreaInput(settings.getString("mail.body", ""), 10000);
-    mailbody.setHeight(200);
-    mailbody.setName("Text");
-    return mailbody;
-  }
-
   public DateInput getAusfuehrungsdatum()
   {
     if (ausfuehrungsdatum != null)
@@ -204,14 +115,15 @@ public class PreNotificationControl extends AbstractControl
     {
       return ct1ausgabe;
     }
-    Abrechnungsausgabe aus = Abrechnungsausgabe.getByKey(
-        settings.getInt("ct1ausgabe", Abrechnungsausgabe.SEPA_DATEI.getKey()));
-    if (aus != Abrechnungsausgabe.SEPA_DATEI
-        && aus != Abrechnungsausgabe.HIBISCUS)
+    Ct1Ausgabe aus = Ct1Ausgabe.getByKey(
+        settings.getInt(settingsprefix + "ct1ausgabe", 
+            Ct1Ausgabe.SEPA_DATEI.getKey()));
+    if (aus != Ct1Ausgabe.SEPA_DATEI
+        && aus != Ct1Ausgabe.HIBISCUS)
     {
-      aus = Abrechnungsausgabe.HIBISCUS;
+      aus = Ct1Ausgabe.HIBISCUS;
     }
-    ct1ausgabe = new SelectInput(Abrechnungsausgabe.values(), aus);
+    ct1ausgabe = new SelectInput(Ct1Ausgabe.values(), aus);
     ct1ausgabe.setName("Ausgabe");
     return ct1ausgabe;
   }
@@ -230,7 +142,7 @@ public class PreNotificationControl extends AbstractControl
 
   public Button getStartButton(final Object currentObject)
   {
-    Button button = new Button("starten", new Action()
+    Button button = new Button("Starten", new Action()
     {
 
       @Override
@@ -238,27 +150,39 @@ public class PreNotificationControl extends AbstractControl
       {
         try
         {
+          saveDruckMailSettings();
+          Object object = currentObject;
+          if (object == null)
+          {
+            if (abrechnungslaufausw != null && abrechnungslaufausw.getValue() != null)
+            {
+              object = abrechnungslaufausw.getValue();
+            }
+            else
+            {
+              GUI.getStatusBar().setErrorText(
+                  "Kein Abrechnungslauf oder keine Lastschrift ausgewhlt");
+              return;
+            }
+          }
           String val = (String) getOutput().getValue();
           String pdfMode = (String) getPdfModus().getValue();
 
-          settings.setAttribute("output", val);
-          settings.setAttribute("pdfModus", pdfMode);
-          settings.setAttribute("mail.subject",
-              (String) mailsubject.getValue());
-          settings.setAttribute("mail.body", (String) mailbody.getValue());
-          settings.setAttribute("tab.selection", folder.getSelectionIndex());
+          settings.setAttribute(settingsprefix + "tab.selection", 
+              folder.getSelectionIndex());
+          saveDruckMailSettings();
 
           if (val.equals(PDF1))
           {
-            generierePDF(currentObject, false, pdfMode);
+            generierePDF(object, false, pdfMode);
           }
           if (val.equals(PDF2))
           {
-            generierePDF(currentObject, true, pdfMode);
+            generierePDF(object, true, pdfMode);
           }
           if (val.equals(EMAIL))
           {
-            generiereEMail(currentObject);
+            generiereEMail(object);
           }
         }
         catch (Exception e)
@@ -273,7 +197,7 @@ public class PreNotificationControl extends AbstractControl
 
   public Button getStart1ctUeberweisungButton(final Object currentObject)
   {
-    Button button = new Button("starten", new Action()
+    Button button = new Button("Starten", new Action()
     {
 
       @Override
@@ -281,20 +205,40 @@ public class PreNotificationControl extends AbstractControl
       {
         try
         {
-          Abrechnungsausgabe aa = (Abrechnungsausgabe) ct1ausgabe.getValue();
-          settings.setAttribute("ct1ausgabe", aa.getKey());
+          saveDruckMailSettings();
+          Object object = currentObject;
+          if (object == null)
+          {
+            if (abrechnungslaufausw != null && abrechnungslaufausw.getValue() != null)
+            {
+              object = abrechnungslaufausw.getValue();
+            }
+            else
+            {
+              GUI.getStatusBar().setErrorText(
+                  "Kein Abrechnungslauf oder keine Lastschrift ausgewhlt");
+              return;
+            }
+          }
+          Ct1Ausgabe aa = (Ct1Ausgabe) ct1ausgabe.getValue();
+          settings.setAttribute(settingsprefix + "ct1ausgabe", aa.getKey());
           if (ausfuehrungsdatum.getValue() == null)
           {
             GUI.getStatusBar().setErrorText("Ausfhrungsdatum fehlt");
             return;
           }
           Date d = (Date) ausfuehrungsdatum.getValue();
-          settings.setAttribute("faelligkeitsdatum",
+          settings.setAttribute(settingsprefix + "faelligkeitsdatum",
               new JVDateFormatDATETIME().format(d));
-          settings.setAttribute("verwendungszweck",
+          settings.setAttribute(settingsprefix + "verwendungszweck",
               (String) getVerwendungszweck().getValue());
-          settings.setAttribute("tab.selection", folder.getSelectionIndex());
-          generiere1ct(currentObject);
+          settings.setAttribute(settingsprefix + "tab.selection", 
+              folder.getSelectionIndex());
+          generiere1ct(object);
+        }
+        catch (OperationCanceledException oce)
+        {
+          throw oce;
         }
         catch (Exception e)
         {
@@ -309,6 +253,89 @@ public class PreNotificationControl extends AbstractControl
   private void generierePDF(Object currentObject, boolean mitMail,
       String pdfMode) throws IOException
   {
+    ArrayList<Lastschrift> lastschriften = new ArrayList<>();
+    if (currentObject instanceof Abrechnungslauf)
+    {
+      Abrechnungslauf abrl = (Abrechnungslauf) currentObject;
+      DBIterator<Lastschrift> it = Einstellungen.getDBService()
+          .createList(Lastschrift.class);
+      it.addFilter("abrechnungslauf = ?", abrl.getID());
+      if (!mitMail)
+      {
+        it.addFilter("(email is null or length(email)=0)");
+      }
+      it.setOrder("order by name, vorname");
+      while (it.hasNext())
+      {
+        lastschriften.add((Lastschrift) it.next());
+      }
+      
+      if (lastschriften.size() == 0 && !mitMail)
+      {
+        GUI.getStatusBar().setErrorText(
+            "Der Abrechnungslauf hat keine Lastschriften ohne Mailadresse");
+        return;
+      }
+      if (lastschriften.size() == 0)
+      {
+        GUI.getStatusBar().setErrorText(
+            "Der Abrechnungslauf hat keine Lastschriften");
+        return;
+      }
+    }
+    else if (currentObject instanceof Lastschrift)
+    {
+      Lastschrift lastschrift = (Lastschrift) currentObject;
+      Abrechnungslauf abrl = (Abrechnungslauf) lastschrift.getAbrechnungslauf();
+      if (abrl.getAbgeschlossen())
+      {
+        GUI.getStatusBar().setErrorText(
+            "Die ausgewhlte Lastschrift ist bereits abgeschlossen");
+        return;
+      }
+      if (!mitMail && lastschrift.getEmail() != null && !lastschrift.getEmail().isEmpty())
+      {
+        GUI.getStatusBar().setErrorText(
+            "Die ausgewhlte Lastschrift hat eine Mail Adresse");
+        return;
+      }
+      else
+      {
+        lastschriften.add(lastschrift);
+      }
+    }
+    else if (currentObject instanceof Lastschrift[])
+    {
+      Lastschrift[] lastschriftarray = (Lastschrift[]) currentObject;
+      for (Lastschrift lastschrift : lastschriftarray)
+      {
+        Abrechnungslauf abrl = (Abrechnungslauf) lastschrift.getAbrechnungslauf();
+        if (abrl.getAbgeschlossen())
+        {
+          GUI.getStatusBar().setErrorText(
+              "Die ausgewhlte Lastschrift mit der Nr " + lastschrift.getID() 
+              + " ist bereits abgeschlossen");
+          return;
+        }
+        if (!(!mitMail && lastschrift.getEmail() != null && !lastschrift.getEmail().isEmpty()))
+        {
+          lastschriften.add(lastschrift);
+        }
+      }
+      if (lastschriften.size() == 0)
+      {
+        GUI.getStatusBar().setErrorText(
+            "Alle ausgewhlten Lastschriften haben eine Mail Adresse");
+        return;
+      }
+    }
+    else
+    {
+      GUI.getStatusBar().setErrorText(
+          "Kein Abrechnungslauf oder keine Lastschrift ausgewhlt");
+      return;
+    }
+    
     boolean einzelnePdfs = false;
     if (pdfMode.equals(EINZELN_NUMMERIERT)
         || pdfMode.equals(EINZELN_MITGLIEDSNUMMER)
@@ -317,7 +344,6 @@ public class PreNotificationControl extends AbstractControl
       einzelnePdfs = true;
     }
 
-    Abrechnungslauf abrl = (Abrechnungslauf) currentObject;
     FileDialog fd = new FileDialog(GUI.getShell(), SWT.SAVE);
     fd.setText("Ausgabedatei whlen.");
     String path = settings.getString("lastdir",
@@ -327,17 +353,17 @@ public class PreNotificationControl extends AbstractControl
       fd.setFilterPath(path);
     }
     fd.setFileName(new Dateiname("prenotification", "",
-        Einstellungen.getEinstellung().getDateinamenmuster(), "PDF").get());
-    fd.setFilterExtensions(new String[] { "*.PDF" });
+        Einstellungen.getEinstellung().getDateinamenmuster(), "pdf").get());
+    fd.setFilterExtensions(new String[] { "*.pdf" });
 
     String s = fd.open();
     if (s == null || s.length() == 0)
     {
       return;
     }
-    if (!s.endsWith(".PDF"))
+    if (!s.toLowerCase().endsWith(".pdf"))
     {
-      s = s + ".PDF";
+      s = s + ".pdf";
     }
     final File file = new File(s);
     settings.setAttribute("lastdir", file.getParent());
@@ -353,23 +379,13 @@ public class PreNotificationControl extends AbstractControl
     {
       fa = new FormularAufbereitung(file);
     }
-    DBIterator<Lastschrift> it = Einstellungen.getDBService()
-        .createList(Lastschrift.class);
-    it.addFilter("abrechnungslauf = ?", abrl.getID());
-    if (!mitMail)
-    {
-      it.addFilter("(email is null or length(email)=0)");
-    }
-    it.setOrder("order by name, vorname");
-
+    
     int dateinummer = 0;
-    String postfix = ".PDF";
+    String postfix = ".pdf";
     String prefix = s.substring(0, s.length() - postfix.length());
 
-    while (it.hasNext())
+    for (Lastschrift ls : lastschriften)
     {
-      Lastschrift ls = (Lastschrift) it.next();
-
       if (einzelnePdfs)
       {
         // schalte Dateinamen um
@@ -406,11 +422,65 @@ public class PreNotificationControl extends AbstractControl
 
   private void generiere1ct(Object currentObject) throws Exception
   {
-    Abrechnungslauf abrl = (Abrechnungslauf) currentObject;
+    ArrayList<Lastschrift> lastschriften = new ArrayList<>();
+    if (currentObject instanceof Abrechnungslauf)
+    {
+      Abrechnungslauf abrl = (Abrechnungslauf) currentObject;
+      DBIterator<Lastschrift> it = Einstellungen.getDBService()
+          .createList(Lastschrift.class);
+      it.addFilter("abrechnungslauf = ?", abrl.getID());
+      it.setOrder("order by name, vorname");
+      while (it.hasNext())
+      {
+        lastschriften.add((Lastschrift) it.next());
+      }
+      if (lastschriften.size() == 0)
+      {
+        GUI.getStatusBar().setErrorText(
+            "Der Abrechnungslauf hat keine Lastschriften");
+        return;
+      }
+    }
+    else if (currentObject instanceof Lastschrift)
+    {
+      Lastschrift lastschrift = (Lastschrift) currentObject;
+      Abrechnungslauf abrl = (Abrechnungslauf) lastschrift.getAbrechnungslauf();
+      if (abrl.getAbgeschlossen())
+      {
+        GUI.getStatusBar().setErrorText(
+            "Die ausgewhlte Lastschrift ist bereits abgeschlossen");
+        return;
+      }
+      lastschriften.add((Lastschrift) currentObject);
+    }
+    else if (currentObject instanceof Lastschrift[])
+    {
+      Lastschrift[] lastschriftarray = (Lastschrift[]) currentObject;
+      for (Lastschrift lastschrift : lastschriftarray)
+      {
+        Abrechnungslauf abrl = (Abrechnungslauf) lastschrift.getAbrechnungslauf();
+        if (abrl.getAbgeschlossen())
+        {
+          GUI.getStatusBar().setErrorText(
+              "Die ausgewhlte Lastschrift mit der Nr " + lastschrift.getID() 
+              + " ist bereits abgeschlossen");
+          return;
+        }
+        lastschriften.add(lastschrift);
+      }
+    }
+    else
+    {
+      GUI.getStatusBar().setErrorText(
+          "Kein Abrechnungslauf oder keine Lastschrift ausgewhlt");
+      return;
+    }
+
     File file = null;
-    Abrechnungsausgabe aa = Abrechnungsausgabe.getByKey(
-        settings.getInt("ct1ausgabe", Abrechnungsausgabe.SEPA_DATEI.getKey()));
-    if (aa == Abrechnungsausgabe.SEPA_DATEI)
+    Ct1Ausgabe aa = Ct1Ausgabe.getByKey(
+        settings.getInt(settingsprefix + "ct1ausgabe", 
+            Ct1Ausgabe.SEPA_DATEI.getKey()));
+    if (aa == Ct1Ausgabe.SEPA_DATEI)
     {
       FileDialog fd = new FileDialog(GUI.getShell(), SWT.SAVE);
       fd.setText("SEPA-Ausgabedatei whlen.");
@@ -421,44 +491,111 @@ public class PreNotificationControl extends AbstractControl
         fd.setFilterPath(path);
       }
       fd.setFileName(new Dateiname("1ctueberweisung", "",
-          Einstellungen.getEinstellung().getDateinamenmuster(), "XML").get());
-      fd.setFilterExtensions(new String[] { "*.XML" });
+          Einstellungen.getEinstellung().getDateinamenmuster(), "xml").get());
+      fd.setFilterExtensions(new String[] { "*.xml" });
 
       String s = fd.open();
       if (s == null || s.length() == 0)
       {
         return;
       }
-      settings.setAttribute("ausgabedateiname", s);
-      if (!s.endsWith(".XML"))
+      settings.setAttribute(settingsprefix + "ausgabedateiname", s);
+      if (!s.toLowerCase().endsWith(".xml"))
       {
-        s = s + ".XML";
+        s = s + ".xml";
       }
       file = new File(s);
       settings.setAttribute("lastdir", file.getParent());
     }
-    String faelligkeitsdatum = settings.getString("faelligkeitsdatum", null);
+    String faelligkeitsdatum = settings.getString(settingsprefix + "faelligkeitsdatum", null);
     Date faell = Datum.toDate(faelligkeitsdatum);
-    Abrechnungsausgabe ct1ausgabe = Abrechnungsausgabe.getByKey(
-        settings.getInt("ct1ausgabe", Abrechnungsausgabe.SEPA_DATEI.getKey()));
-    String verwendungszweck = settings.getString("verwendungszweck", "");
+    Ct1Ausgabe ct1ausgabe = Ct1Ausgabe.getByKey(
+        settings.getInt(settingsprefix + "ct1ausgabe", Ct1Ausgabe.SEPA_DATEI.getKey()));
+    String verwendungszweck = settings.getString(settingsprefix + "verwendungszweck", "");
     Ct1Ueberweisung ct1ueberweisung = new Ct1Ueberweisung();
-    int anzahl = ct1ueberweisung.write(abrl, file, faell, ct1ausgabe,
+    int anzahl = ct1ueberweisung.write(lastschriften, file, faell, ct1ausgabe,
         verwendungszweck);
     GUI.getStatusBar().setSuccessText("Anzahl berweisungen: " + anzahl);
   }
 
   private void generiereEMail(Object currentObject) throws IOException
   {
-    Abrechnungslauf abrl = (Abrechnungslauf) currentObject;
-    DBIterator<Lastschrift> it = Einstellungen.getDBService()
-        .createList(Lastschrift.class);
-    it.addFilter("abrechnungslauf = ?", abrl.getID());
-    it.addFilter("email is not null and length(email) > 0");
-    it.setOrder("order by name, vorname");
-    String betr = (String) getMailSubject().getValue();
-    String text = (String) getMailBody().getValue();
-    sendeMail(it, betr, text);
+    ArrayList<Lastschrift> lastschriften = new ArrayList<>();
+    if (currentObject instanceof Abrechnungslauf)
+    {
+      Abrechnungslauf abrl = (Abrechnungslauf) currentObject;
+      DBIterator<Lastschrift> it = Einstellungen.getDBService()
+          .createList(Lastschrift.class);
+      it.addFilter("abrechnungslauf = ?", abrl.getID());
+      it.addFilter("email is not null and length(email) > 0");
+      it.setOrder("order by name, vorname");
+      while (it.hasNext())
+      {
+        lastschriften.add((Lastschrift) it.next());
+      }
+      if (lastschriften.size() == 0)
+      {
+        GUI.getStatusBar().setErrorText(
+            "Der Abrechnungslauf hat keine Lastschriften mit Mail Adresse");
+        return;
+      }
+    }
+    else if (currentObject instanceof Lastschrift)
+    {
+      Lastschrift lastschrift = (Lastschrift) currentObject;
+      Abrechnungslauf abrl = (Abrechnungslauf) lastschrift.getAbrechnungslauf();
+      if (abrl.getAbgeschlossen())
+      {
+        GUI.getStatusBar().setErrorText(
+            "Die ausgewhlte Lastschrift ist bereits abgeschlossen");
+        return;
+      }
+      if (lastschrift.getEmail() != null && !lastschrift.getEmail().isEmpty())
+      {
+        lastschriften.add(lastschrift);
+      }
+      else
+      {
+        GUI.getStatusBar().setErrorText(
+            "Die ausgewhlte Lastschrift hat keine Mail Adresse");
+        return;
+      }
+    }
+    else if (currentObject instanceof Lastschrift[])
+    {
+      Lastschrift[] lastschriftarray = (Lastschrift[]) currentObject;
+      for (Lastschrift lastschrift : lastschriftarray)
+      {
+        Abrechnungslauf abrl = (Abrechnungslauf) lastschrift.getAbrechnungslauf();
+        if (abrl.getAbgeschlossen())
+        {
+          GUI.getStatusBar().setErrorText(
+              "Die ausgewhlte Lastschrift mit der Nr " + lastschrift.getID() 
+              + " ist bereits abgeschlossen");
+          return;
+        }
+        if (lastschrift.getEmail() != null && !lastschrift.getEmail().isEmpty())
+        {
+          lastschriften.add(lastschrift);
+        }
+      }
+      if (lastschriften.size() == 0)
+      {
+        GUI.getStatusBar().setErrorText(
+            "Keine der ausgewhlten Lastschriften hat eine Mail Adresse");
+        return;
+      }
+    }
+    else
+    {
+      GUI.getStatusBar().setErrorText(
+          "Kein Abrechnungslauf oder keine Lastschrift ausgewhlt");
+      return;
+    }
+
+    String betr = (String) getBetreff().getValue();
+    String text = (String) getTxt().getValue();
+    sendeMail(lastschriften, betr, text);
   }
 
   private void aufbereitenFormular(Lastschrift ls, Formular fo)
@@ -469,13 +606,15 @@ public class PreNotificationControl extends AbstractControl
     fa.writeForm(fo, map);
   }
 
-  private void sendeMail(final DBIterator<Lastschrift> it, final String betr,
+  private void sendeMail(final ArrayList<Lastschrift> lastschriften, final String betr,
       final String txt) throws RemoteException
   {
 
     BackgroundTask t = new BackgroundTask()
     {
 
+      private boolean cancel = false;
+
       @Override
       public void run(ProgressMonitor monitor)
       {
@@ -500,9 +639,18 @@ public class PreNotificationControl extends AbstractControl
           monitor.setStatus(ProgressMonitor.STATUS_RUNNING);
           monitor.setPercentComplete(0);
           int sentCount = 0;
-          while (it.hasNext())
+          int zae = 0;
+          int size = lastschriften.size();
+          for (Lastschrift ls : lastschriften)
           {
-            Lastschrift ls = (Lastschrift) it.next();
+            if(isInterrupted())
+            {
+              monitor.setStatus(ProgressMonitor.STATUS_ERROR);
+              monitor.setStatusText("Mailversand abgebrochen");
+              monitor.setPercentComplete(100);
+              return;
+            }
+            
             VelocityContext context = new VelocityContext();
             context.put("dateformat", new JVDateFormatTTMMJJJJ());
             context.put("decimalformat", Einstellungen.DECIMALFORMAT);
@@ -550,6 +698,9 @@ public class PreNotificationControl extends AbstractControl
               Logger.error("Fehler beim Mailversand", e);
               monitor.log(ls.getEmail() + " - " + e.getMessage());
             }
+            zae++;
+            double proz = (double) zae / (double) size * 100d;
+            monitor.setPercentComplete((int) proz);
           }
           monitor.setPercentComplete(100);
           monitor.setStatus(ProgressMonitor.STATUS_DONE);
@@ -557,12 +708,6 @@ public class PreNotificationControl extends AbstractControl
               String.format("Anzahl verschickter Mails: %d", sentCount));
           GUI.getStatusBar().setSuccessText(
               "Mail" + (sentCount > 1 ? "s" : "") + " verschickt");
-          GUI.getCurrentView().reload();
-        }
-        catch (ApplicationException ae)
-        {
-          Logger.error("", ae);
-          monitor.log(ae.getMessage());
         }
         catch (Exception re)
         {
@@ -574,13 +719,13 @@ public class PreNotificationControl extends AbstractControl
       @Override
       public void interrupt()
       {
-        //
+        this.cancel = true;
       }
 
       @Override
       public boolean isInterrupted()
       {
-        return false;
+        return this.cancel;
       }
     };
     Application.getController().start(t);
