@@ -17,40 +17,55 @@
 package de.jost_net.JVerein.gui.control;
 
 import java.io.File;
-import java.io.IOException;
+import java.io.FileInputStream;
+import java.io.StringWriter;
 import java.rmi.RemoteException;
+import java.sql.ResultSet;
+import java.sql.SQLException;
+import java.sql.Timestamp;
+import java.util.ArrayList;
 import java.util.Date;
+import java.util.List;
 import java.util.Map;
+import java.util.TreeSet;
 
-import org.eclipse.swt.SWT;
+import org.apache.velocity.VelocityContext;
+import org.apache.velocity.app.Velocity;
 import org.eclipse.swt.widgets.Event;
-import org.eclipse.swt.widgets.FileDialog;
 import org.eclipse.swt.widgets.Listener;
 
 import de.jost_net.JVerein.Einstellungen;
-import de.jost_net.JVerein.Queries.SpendenbescheinigungBuchungQuery;
 import de.jost_net.JVerein.Variable.AllgemeineMap;
+import de.jost_net.JVerein.Variable.MitgliedMap;
+import de.jost_net.JVerein.Variable.VarTools;
 import de.jost_net.JVerein.gui.action.SpendenbescheinigungAction;
 import de.jost_net.JVerein.gui.action.SpendenbescheinigungPrintAction;
 import de.jost_net.JVerein.gui.formatter.BuchungsartFormatter;
+import de.jost_net.JVerein.gui.formatter.JaNeinFormatter;
 import de.jost_net.JVerein.gui.formatter.MitgliedskontoFormatter;
 import de.jost_net.JVerein.gui.input.FormularInput;
+import de.jost_net.JVerein.gui.input.MailAuswertungInput;
 import de.jost_net.JVerein.gui.menu.SpendenbescheinigungMenu;
 import de.jost_net.JVerein.gui.parts.BuchungListTablePart;
-import de.jost_net.JVerein.io.FileViewer;
-import de.jost_net.JVerein.io.FormularAufbereitung;
+import de.jost_net.JVerein.gui.view.SpendenbescheinigungMailView;
+import de.jost_net.JVerein.io.MailSender;
+import de.jost_net.JVerein.keys.Adressblatt;
+import de.jost_net.JVerein.keys.Ausgabeart;
 import de.jost_net.JVerein.keys.FormularArt;
 import de.jost_net.JVerein.keys.HerkunftSpende;
 import de.jost_net.JVerein.keys.Spendenart;
 import de.jost_net.JVerein.rmi.Buchung;
 import de.jost_net.JVerein.rmi.Formular;
 import de.jost_net.JVerein.rmi.Konto;
+import de.jost_net.JVerein.rmi.Mail;
+import de.jost_net.JVerein.rmi.MailAnhang;
+import de.jost_net.JVerein.rmi.MailEmpfaenger;
+import de.jost_net.JVerein.rmi.Mitglied;
 import de.jost_net.JVerein.rmi.Spendenbescheinigung;
 import de.jost_net.JVerein.util.Dateiname;
 import de.jost_net.JVerein.util.JVDateFormatTTMMJJJJ;
-import de.willuhn.datasource.rmi.DBIterator;
 import de.willuhn.datasource.rmi.DBService;
-import de.willuhn.jameica.gui.AbstractControl;
+import de.willuhn.datasource.rmi.ResultSetExtractor;
 import de.willuhn.jameica.gui.AbstractView;
 import de.willuhn.jameica.gui.Action;
 import de.willuhn.jameica.gui.GUI;
@@ -65,14 +80,17 @@ import de.willuhn.jameica.gui.input.SelectInput;
 import de.willuhn.jameica.gui.input.TextInput;
 import de.willuhn.jameica.gui.parts.Button;
 import de.willuhn.jameica.gui.parts.TablePart;
+import de.willuhn.jameica.gui.parts.table.FeatureSummary;
+import de.willuhn.jameica.system.Application;
+import de.willuhn.jameica.system.BackgroundTask;
 import de.willuhn.logging.Logger;
 import de.willuhn.util.ApplicationException;
+import de.willuhn.util.ProgressMonitor;
 
-public class SpendenbescheinigungControl extends AbstractControl
+public class SpendenbescheinigungControl extends DruckMailControl
 {
 
-  private de.willuhn.jameica.system.Settings settings;
-
+  // Spendenbescheinigung View
   private TablePart spbList;
 
   private SelectInput spendenart;
@@ -109,9 +127,12 @@ public class SpendenbescheinigungControl extends AbstractControl
 
   private TablePart buchungsList;
 
-  private SpendenbescheinigungBuchungQuery query;
-
   private Spendenbescheinigung spendenbescheinigung;
+  
+  private boolean and = false;
+
+  private String sql = "";
+  
 
   public SpendenbescheinigungControl(AbstractView view)
   {
@@ -173,7 +194,7 @@ public class SpendenbescheinigungControl extends AbstractControl
     {
       return zeile1;
     }
-    zeile1 = new TextInput(getSpendenbescheinigung().getZeile1(), 40);
+    zeile1 = new TextInput(getSpendenbescheinigung().getZeile1(), 80);
     if (withFocus)
     {
       zeile1.focus();
@@ -187,7 +208,7 @@ public class SpendenbescheinigungControl extends AbstractControl
     {
       return zeile2;
     }
-    zeile2 = new TextInput(getSpendenbescheinigung().getZeile2(), 40);
+    zeile2 = new TextInput(getSpendenbescheinigung().getZeile2(), 80);
     return zeile2;
   }
 
@@ -197,7 +218,7 @@ public class SpendenbescheinigungControl extends AbstractControl
     {
       return zeile3;
     }
-    zeile3 = new TextInput(getSpendenbescheinigung().getZeile3(), 40);
+    zeile3 = new TextInput(getSpendenbescheinigung().getZeile3(), 80);
     return zeile3;
   }
 
@@ -207,7 +228,7 @@ public class SpendenbescheinigungControl extends AbstractControl
     {
       return zeile4;
     }
-    zeile4 = new TextInput(getSpendenbescheinigung().getZeile4(), 40);
+    zeile4 = new TextInput(getSpendenbescheinigung().getZeile4(), 80);
     return zeile4;
   }
 
@@ -217,7 +238,7 @@ public class SpendenbescheinigungControl extends AbstractControl
     {
       return zeile5;
     }
-    zeile5 = new TextInput(getSpendenbescheinigung().getZeile5(), 40);
+    zeile5 = new TextInput(getSpendenbescheinigung().getZeile5(), 80);
     return zeile5;
   }
 
@@ -227,7 +248,7 @@ public class SpendenbescheinigungControl extends AbstractControl
     {
       return zeile6;
     }
-    zeile6 = new TextInput(getSpendenbescheinigung().getZeile6(), 40);
+    zeile6 = new TextInput(getSpendenbescheinigung().getZeile6(), 80);
     return zeile6;
   }
 
@@ -237,7 +258,7 @@ public class SpendenbescheinigungControl extends AbstractControl
     {
       return zeile7;
     }
-    zeile7 = new TextInput(getSpendenbescheinigung().getZeile7(), 40);
+    zeile7 = new TextInput(getSpendenbescheinigung().getZeile7(), 80);
     return zeile7;
   }
 
@@ -293,6 +314,7 @@ public class SpendenbescheinigungControl extends AbstractControl
     {
       formular = new FormularInput(FormularArt.SPENDENBESCHEINIGUNG, def);
     }
+    formular.setPleaseChoose("Standard");
     return formular;
   }
 
@@ -302,8 +324,29 @@ public class SpendenbescheinigungControl extends AbstractControl
     {
       return ersatzaufwendungen;
     }
-    ersatzaufwendungen = new CheckboxInput(
-        getSpendenbescheinigung().getErsatzAufwendungen());
+    List<Buchung> buchungen = getSpendenbescheinigung().getBuchungen();
+    boolean check = false;
+    if (buchungen != null && buchungen.size() == 1)
+    {
+      // Es ist keine Sachspende und keine Sammelspendenbescheinigung
+      if (getSpendenbescheinigung().getAutocreate())
+      {
+        // Verzicht aus Buchung lesen
+        check = buchungen.get(0).getVerzicht();
+      }
+      else
+      {
+        // Wegen Kompabilitt zu frher
+        check = getSpendenbescheinigung().getErsatzAufwendungen();
+      }
+    }
+    ersatzaufwendungen = new CheckboxInput(check);
+    if (buchungen != null && buchungen.size() > 1)
+    {
+      // Sammelspendenbescheinigung
+      ersatzaufwendungen.setName("*siehe Buchungsliste");
+    }
+    ersatzaufwendungen.disable();
     return ersatzaufwendungen;
   }
 
@@ -345,12 +388,11 @@ public class SpendenbescheinigungControl extends AbstractControl
 
   public Part getBuchungsList() throws RemoteException
   {
-    query = new SpendenbescheinigungBuchungQuery(getSpendenbescheinigung());
+    Spendenbescheinigung spb = getSpendenbescheinigung();
     if (buchungsList == null)
     {
-      // buchungsList = new BuchungListTablePart(query.get(), new
-      // BuchungAction());
-      buchungsList = new BuchungListTablePart(query.get(), null);
+
+      buchungsList = new BuchungListTablePart(spb.getBuchungen(), null);
       buchungsList.addColumn("Nr", "id-int");
       buchungsList.addColumn("Konto", "konto", new Formatter()
       {
@@ -401,18 +443,18 @@ public class SpendenbescheinigungControl extends AbstractControl
           new CurrencyFormatter("", Einstellungen.DECIMALFORMAT));
       buchungsList.addColumn("Mitglied", "mitgliedskonto",
           new MitgliedskontoFormatter());
+      buchungsList.addColumn("Ersatz fr Aufwendungen", "verzicht", new JaNeinFormatter());
       buchungsList.setMulti(true);
       // buchungsList.setContextMenu(new BuchungMenu(this));
       buchungsList.setRememberColWidths(true);
       buchungsList.setRememberOrder(true);
       buchungsList.setRememberState(true);
-      buchungsList.setSummary(true);
+      buchungsList.addFeature(new FeatureSummary());
     }
     else
     {
       buchungsList.removeAll();
-
-      for (Buchung bu : query.get())
+      for (Buchung bu : spb.getBuchungen())
       {
         buchungsList.addItem(bu);
       }
@@ -465,68 +507,24 @@ public class SpendenbescheinigungControl extends AbstractControl
     }
   }
 
-  public Button getPDFStandardButton()
+  public Button getDruckUndMailButton()
   {
-    Button b = new Button("PDF (Standard)", new Action()
+  
+    Button b = new Button("Druck und Mail", new Action()
     {
 
-      /**
-       * Diese Action verwendet die "SpendenbescheinigungPrintAction" fr die
-       * Aufbereitung des Dokumentes. Als Rahmen ist der Dialog zur Dateiauswahl
-       * und die Anzeige des Dokumentes um die Generierung gesetzt.
-       */
       @Override
       public void handleAction(Object context) throws ApplicationException
       {
+        Spendenbescheinigung spb = getSpendenbescheinigung();
         try
         {
-          Spendenbescheinigung spb = getSpendenbescheinigung();
           if (spb.isNewObject())
           {
             GUI.getStatusBar()
-                .setErrorText("Spendenbescheinigung bitte erst speichern!");
+            .setErrorText("Spendenbescheinigung bitte erst speichern!");
             return;
           }
-          FileDialog fd = new FileDialog(GUI.getShell(), SWT.SAVE);
-          fd.setText("Ausgabedatei whlen.");
-          String path = Einstellungen.getEinstellung()
-              .getSpendenbescheinigungverzeichnis();
-          if (path != null && path.length() > 0)
-          {
-            fd.setFilterPath(path);
-          }
-          if (spb.getMitglied() != null)
-          {
-            fd.setFileName(new Dateiname(spb.getMitglied(),
-                spb.getBescheinigungsdatum(), "Spendenbescheinigung",
-                Einstellungen.getEinstellung().getDateinamenmusterSpende(),
-                "pdf").get());
-          }
-          else
-          {
-            fd.setFileName(new Dateiname(spb.getZeile1(), spb.getZeile2(),
-                spb.getBescheinigungsdatum(), "Spendenbescheinigung",
-                Einstellungen.getEinstellung().getDateinamenmusterSpende(),
-                "pdf").get());
-          }
-          fd.setFilterExtensions(new String[] { "*.pdf" });
-
-          String s = fd.open();
-          if (s == null || s.length() == 0)
-          {
-            return;
-          }
-          if (!s.endsWith(".pdf"))
-          {
-            s = s + ".pdf";
-          }
-          final File file = new File(s);
-          //
-          SpendenbescheinigungPrintAction spa = new SpendenbescheinigungPrintAction(
-              true, s);
-          spa.handleAction(spb);
-          GUI.getStatusBar().setSuccessText("Spendenbescheinigung erstellt");
-          FileViewer.show(file);
         }
         catch (RemoteException e)
         {
@@ -534,112 +532,22 @@ public class SpendenbescheinigungControl extends AbstractControl
           throw new ApplicationException(
               "Fehler bei der Aufbereitung der Spendenbescheinigung");
         }
+        GUI.startView(SpendenbescheinigungMailView.class, 
+            new Spendenbescheinigung[] { (Spendenbescheinigung) spb });
       }
-    }, getSpendenbescheinigung(), false, "file-pdf.png");
+    }, getSpendenbescheinigung(), false, "document-print.png");
     return b;
   }
 
-  public Button getPDFIndividuellButton()
-  {
-    Button b = new Button("PDF (Individuell)", new Action()
-    {
-
-      @Override
-      public void handleAction(Object context) throws ApplicationException
-      {
-        try
-        {
-          generiereSpendenbescheinigungIndividuell();
-        }
-        catch (RemoteException e)
-        {
-          Logger.error(e.getMessage());
-          throw new ApplicationException(
-              "Fehler bei der Aufbereitung der Spendenbescheinigung");
-        }
-        catch (IOException e)
-        {
-          Logger.error(e.getMessage());
-          throw new ApplicationException(
-              "Fehler bei der Aufbereitung der Spendenbescheinigung");
-        }
-      }
-    }, null, false, "file-pdf.png");
-    return b;
-  }
-
-  private void generiereSpendenbescheinigungIndividuell() throws IOException
-  {
-    Spendenbescheinigung spb = getSpendenbescheinigung();
-    if (spb.isNewObject())
-    {
-      GUI.getStatusBar()
-          .setErrorText("Spendenbescheinigung bitte erst speichern!");
-      return;
-    }
-    FileDialog fd = new FileDialog(GUI.getShell(), SWT.SAVE);
-    fd.setText("Ausgabedatei whlen.");
-    String path = Einstellungen.getEinstellung()
-        .getSpendenbescheinigungverzeichnis();
-    if (path != null && path.length() > 0)
-    {
-      fd.setFilterPath(path);
-    }
-    if (spb.getMitglied() != null)
-    {
-      fd.setFileName(new Dateiname(spb.getMitglied(),
-          spb.getBescheinigungsdatum(), "Spendenbescheinigung",
-          Einstellungen.getEinstellung().getDateinamenmusterSpende(), "pdf")
-              .get());
-    }
-    else
-    {
-      fd.setFileName(new Dateiname(spb.getZeile1(), spb.getZeile2(),
-          spb.getBescheinigungsdatum(), "Spendenbescheinigung",
-          Einstellungen.getEinstellung().getDateinamenmusterSpende(), "pdf")
-              .get());
-    }
-    fd.setFilterExtensions(new String[] { "*.pdf" });
-
-    String s = fd.open();
-    if (s == null || s.length() == 0)
-    {
-      return;
-    }
-    if (!s.endsWith(".pdf"))
-    {
-      s = s + ".pdf";
-    }
-    final File file = new File(s);
-    settings.setAttribute("lastdir", file.getParent());
-
-    /* Check ob auch ein Formular ausgewaehlt ist */
-    Formular spendeformular = getSpendenbescheinigung().getFormular();
-    if (spendeformular == null)
-    {
-      GUI.getStatusBar().setErrorText("Bitte Formular auswaehlen");
-      return;
-    }
-
-    Formular fo = (Formular) Einstellungen.getDBService()
-        .createObject(Formular.class, spendeformular.getID());
-    Map<String, Object> map = getSpendenbescheinigung().getMap(null);
-    map = new AllgemeineMap().getMap(map);
-    FormularAufbereitung fa = new FormularAufbereitung(file);
-    fa.writeForm(fo, map);
-    fa.showFormular();
-
-  }
-
   public Part getSpendenbescheinigungList() throws RemoteException
   {
-    DBService service = Einstellungen.getDBService();
-    DBIterator<Spendenbescheinigung> spendenbescheinigungen = service
-        .createList(Spendenbescheinigung.class);
-    spendenbescheinigungen.setOrder("ORDER BY bescheinigungsdatum desc");
-
-    spbList = new TablePart(spendenbescheinigungen,
-        new SpendenbescheinigungAction());
+    if (spbList != null)
+    {
+      return spbList;
+    }
+    spbList = new TablePart(getSpendenbescheinigungen(),
+        new SpendenbescheinigungAction(Spendenart.SACHSPENDE));
+    spbList.addColumn("Nr", "id-int");
     spbList.addColumn("Bescheinigungsdatum", "bescheinigungsdatum",
         new DateFormatter(new JVDateFormatTTMMJJJJ()));
     spbList.addColumn("Spendedatum", "spendedatum",
@@ -657,21 +565,374 @@ public class SpendenbescheinigungControl extends AbstractControl
     spbList.setRememberColWidths(true);
     spbList.setContextMenu(new SpendenbescheinigungMenu());
     spbList.setRememberOrder(true);
-    spbList.setSummary(false);
+    spbList.addFeature(new FeatureSummary());
     spbList.setMulti(true);
     return spbList;
   }
 
-  public void refreshTable() throws RemoteException
+  public void TabRefresh()
   {
-    spbList.removeAll();
-    DBIterator<Spendenbescheinigung> spendenbescheinigungen = Einstellungen
-        .getDBService().createList(Spendenbescheinigung.class);
-    spendenbescheinigungen.setOrder("ORDER BY bescheinigungsdatum desc");
-    while (spendenbescheinigungen.hasNext())
+    if (spbList != null)
     {
-      spbList.addItem(spendenbescheinigungen.next());
+      try
+      {
+        spbList.removeAll();
+        for (Spendenbescheinigung spb : getSpendenbescheinigungen())
+        {
+          spbList.addItem(spb);
+        }
+      }
+      catch (RemoteException e1)
+      {
+        Logger.error("Fehler", e1);
+      }
     }
   }
 
+  @SuppressWarnings("unchecked")
+  private ArrayList<Spendenbescheinigung> getSpendenbescheinigungen() throws RemoteException
+  {
+    final DBService service = Einstellungen.getDBService();
+    ArrayList<Object> bedingungen = new ArrayList<>();
+    and = false;
+    
+    sql = "select spendenbescheinigung.*  from spendenbescheinigung ";
+    sql +=  "left join mitglied on (spendenbescheinigung.mitglied = mitglied.id) ";
+    
+    if (isSuchnameAktiv() && getSuchname().getValue() != null)
+    {
+      String tmpSuchname = (String) getSuchname().getValue();
+      if (tmpSuchname.length() > 0)
+      {
+        addCondition("(lower(zeile2) like ?)");
+        bedingungen.add("%" + tmpSuchname.toLowerCase() + "%");
+      }
+    }
+    if (isMailauswahlAktiv())
+    {
+      int mailauswahl = (Integer) getMailauswahl().getValue();
+      if (mailauswahl == MailAuswertungInput.OHNE)
+      {
+        addCondition("(email is null or length(email) = 0)");
+      }
+      if (mailauswahl == MailAuswertungInput.MIT)
+      {
+        addCondition("(email is  not null and length(email) > 0)");
+      }
+    }
+    if (isDatumvonAktiv() && getDatumvon().getValue() != null)
+    {
+      addCondition("bescheinigungsdatum >= ?");
+      Date d = (Date) getDatumvon().getValue();
+      bedingungen.add(new java.sql.Date(d.getTime()));
+    }
+    if (isDatumbisAktiv() && getDatumbis().getValue() != null)
+    {
+      addCondition("bescheinigungsdatum <= ?");
+      Date d = (Date) getDatumbis().getValue();
+      bedingungen.add(new java.sql.Date(d.getTime()));
+    }
+    if (isEingabedatumvonAktiv() && getEingabedatumvon().getValue() != null)
+    {
+      addCondition("spendedatum >= ?");
+      Date d = (Date) getEingabedatumvon().getValue();
+      bedingungen.add(new java.sql.Date(d.getTime()));
+    }
+    if (isEingabedatumbisAktiv() && getEingabedatumbis().getValue() != null)
+    {
+      addCondition("spendedatum <= ?");
+      Date d = (Date) getEingabedatumbis().getValue();
+      bedingungen.add(new java.sql.Date(d.getTime()));
+    }
+    sql += " ORDER BY bescheinigungsdatum desc ";
+    
+    ResultSetExtractor rs = new ResultSetExtractor()
+    {
+      @Override
+      public Object extract(ResultSet rs) throws RemoteException, SQLException
+      {
+        ArrayList<Spendenbescheinigung> list = new ArrayList<>();
+        while (rs.next())
+        {
+          list.add(
+              (Spendenbescheinigung) service.createObject(Spendenbescheinigung.class, rs.getString(1)));
+        }
+        return list;
+      }
+    };
+
+    return (ArrayList<Spendenbescheinigung>) service.execute(sql, bedingungen.toArray(),
+        rs);
+  }
+  
+  private void addCondition(String condition)
+  {
+    if (and)
+    {
+      sql += " AND ";
+    }
+    else
+    {
+      sql += "where ";
+    }
+    and = true;
+    sql += condition;
+  }
+  
+  // Mail/Drucken View
+  @Override
+  public String getInfoText(Object spbArray)
+  {
+    Spendenbescheinigung[] spbArr = (Spendenbescheinigung[]) spbArray;
+    String text = "Es wurden " + spbArr.length + 
+        " Spendenbescheinigungen ausgewhlt"
+        + "\nFolgende Mitglieder haben keine Mailadresse:";
+    try
+    {
+      for (Spendenbescheinigung spb: spbArr)
+      {
+        Mitglied m = spb.getMitglied();
+        if (m != null && ( m.getEmail() == null || m.getEmail().isEmpty()))
+        {
+          text = text + "\n - " + m.getName()
+              + ", " + m.getVorname();
+        }
+      }
+      text = text 
+          + "\nFr folgende Spendenbescheinigungen existiert kein Mitglied und keine Mailadresse:";
+      for (Spendenbescheinigung spb: spbArr)
+      {
+        if (spb.getMitglied() == null)
+        {
+          text = text  + "\n - " + spb.getZeile1()
+              + ", " + spb.getZeile2() + ", " + spb.getZeile3();
+        }
+      }
+    }
+    catch (Exception ex)
+    {
+      GUI.getStatusBar().setErrorText("Fehler beim Ermitteln der Mitglieder aus den Spendenbescheinigungen");
+    }
+    return text;
+  }
+  
+  public Button getStartButton(final Object currentObject)
+  {
+    Button button = new Button("Starten", new Action()
+    {
+
+      @Override
+      public void handleAction(Object context)
+      {
+        try
+        {
+          saveDruckMailSettings();
+          Spendenbescheinigung[] spbArray = null;
+          if (currentObject == null)
+          {
+            ArrayList<Spendenbescheinigung> spblist = getSpendenbescheinigungen();
+            if (spblist.size() == 0)
+            {
+              GUI.getStatusBar()
+              .setSuccessText("Fr die gewhlten Filterkriterien wurden "
+                  + "keine Spendenbescheinigungen gefunden");
+              return;
+            }
+            spbArray = spblist.toArray(new Spendenbescheinigung[spblist.size()]);
+          }
+          else if (currentObject instanceof Spendenbescheinigung[])
+          {
+            spbArray = (Spendenbescheinigung[]) currentObject;
+          }
+          else
+          {
+            return;
+          }
+          generatePdf((String) mailtext.getValue(), (Adressblatt) adressblatt.getValue(),
+               spbArray, (Ausgabeart) ausgabeart.getValue());
+          if ((Ausgabeart) ausgabeart.getValue() == Ausgabeart.MAIL)
+          {
+            sendeMail((String) mailbetreff.getValue(),
+                (String) mailtext.getValue(), spbArray);
+          }
+        }
+        catch (Exception e)
+        {
+          Logger.error("Fehler", e);
+          GUI.getStatusBar().setErrorText(e.getMessage());
+        }
+      }
+    }, null, true, "walking.png");
+    return button;
+  }
+
+  private void generatePdf(String text, Adressblatt adressblatt, 
+      Spendenbescheinigung[] spba, Ausgabeart ausgabeart)
+      throws ApplicationException
+  {
+    boolean open = false;
+    if (ausgabeart ==  Ausgabeart.DRUCK)
+      open = true;
+    SpendenbescheinigungPrintAction action = 
+        new SpendenbescheinigungPrintAction(text, adressblatt, open);
+    action.handleAction(spba);
+  }
+  
+  private void sendeMail(final String betr, final String txt,
+      final Spendenbescheinigung[] spba) throws RemoteException
+  {
+
+    BackgroundTask t = new BackgroundTask()
+    {
+
+      private boolean cancel = false;
+
+      @Override
+      public void run(ProgressMonitor monitor)
+      {
+        try
+        {
+          MailSender sender = new MailSender(
+              Einstellungen.getEinstellung().getSmtpServer(),
+              Einstellungen.getEinstellung().getSmtpPort(),
+              Einstellungen.getEinstellung().getSmtpAuthUser(),
+              Einstellungen.getEinstellung().getSmtpAuthPwd(),
+              Einstellungen.getEinstellung().getSmtpFromAddress(),
+              Einstellungen.getEinstellung().getSmtpFromAnzeigename(),
+              Einstellungen.getEinstellung().getMailAlwaysBcc(),
+              Einstellungen.getEinstellung().getMailAlwaysCc(),
+              Einstellungen.getEinstellung().getSmtpSsl(),
+              Einstellungen.getEinstellung().getSmtpStarttls(),
+              Einstellungen.getEinstellung().getMailVerzoegerung(),
+              Einstellungen.getImapCopyData());
+
+          Velocity.init();
+          Logger.debug("preparing velocity context");
+          monitor.setStatus(ProgressMonitor.STATUS_RUNNING);
+          monitor.setPercentComplete(0);
+
+          int sentCount = 0;
+          int size = spba.length;
+          for (int i=0; i < size; i++)
+          {
+            if(isInterrupted())
+            {
+              monitor.setStatus(ProgressMonitor.STATUS_ERROR);
+              monitor.setStatusText("Mailversand abgebrochen");
+              monitor.setPercentComplete(100);
+              return;
+            }
+            
+            double proz = (double) i / (double) size * 100d;
+            monitor.setPercentComplete((int) proz);
+            Mitglied m = spba[i].getMitglied();
+            if (m == null || m.getEmail() == null || m.getEmail().isEmpty())
+            {
+              continue;
+            }
+            VelocityContext context = new VelocityContext();
+            context.put("dateformat", new JVDateFormatTTMMJJJJ());
+            context.put("decimalformat", Einstellungen.DECIMALFORMAT);
+            context.put("email", m.getEmail());
+
+            Map<String, Object> map = new MitgliedMap().getMap(m, null);
+            map = new AllgemeineMap().getMap(map);
+            VarTools.add(context, map);
+
+            StringWriter wtext1 = new StringWriter();
+            Velocity.evaluate(context, wtext1, "LOG", betr);
+
+            StringWriter wtext2 = new StringWriter();
+            Velocity.evaluate(context, wtext2, "LOG", txt);
+
+            try
+            {
+              String path = Einstellungen.getEinstellung()
+                  .getSpendenbescheinigungverzeichnis();
+              if (path == null || path.length() == 0)
+              {
+                path = settings.getString("lastdir", System.getProperty("user.home"));
+              }
+
+              settings.setAttribute("lastdir", path);
+              path = path.endsWith(File.separator) ? path : path + File.separator;
+              TreeSet<MailAnhang> anhang = new TreeSet<MailAnhang>();
+              MailAnhang anh = (MailAnhang) Einstellungen.getDBService()
+                  .createObject(MailAnhang.class, null);
+              String fileName = new Dateiname(m,
+                  spba[i].getBescheinigungsdatum(), "Spendenbescheinigung",
+                  Einstellungen.getEinstellung().getDateinamenmusterSpende(),
+                  "pdf").get();
+              anh.setDateiname(fileName);
+              fileName = path + fileName;
+              File file = new File(fileName);
+              FileInputStream fis = new FileInputStream(file);
+              byte[] buffer = new byte[(int) file.length()];
+              fis.read(buffer);
+              anh.setAnhang(buffer);
+              anhang.add(anh);
+              fis.close();
+
+              sender.sendMail(m.getEmail(), wtext1.getBuffer().toString(),
+                  wtext2.getBuffer().toString(), anhang);
+              sentCount++;
+
+              // Mail in die Datenbank schreiben
+              Mail mail = (Mail) Einstellungen.getDBService()
+                  .createObject(Mail.class, null);
+              Timestamp ts = new Timestamp(new Date().getTime());
+              mail.setBearbeitung(ts);
+              mail.setBetreff(wtext1.getBuffer().toString());
+              mail.setTxt(wtext2.getBuffer().toString());
+              mail.setVersand(ts);
+              mail.store();
+              MailEmpfaenger empf = (MailEmpfaenger) Einstellungen
+                  .getDBService().createObject(MailEmpfaenger.class, null);
+              empf.setMail(mail);
+              empf.setMitglied(m);
+              empf.setVersand(ts);
+              empf.store();
+              anh.setMail(mail);
+              if (Einstellungen.getEinstellung().getAnhangSpeichern())
+              {
+                anh.store();
+              }
+
+              monitor.log(m.getEmail() + " - versendet");
+            }
+            catch (Exception e)
+            {
+              Logger.error("Fehler beim Mailversand", e);
+              monitor.log(m.getEmail() + " - " + e.getMessage());
+            }
+          }
+          monitor.setPercentComplete(100);
+          monitor.setStatus(ProgressMonitor.STATUS_DONE);
+          monitor.setStatusText(
+              String.format("Anzahl verschickter Mails: %d", sentCount));
+          GUI.getStatusBar().setSuccessText(
+              "Mail" + (sentCount > 1 ? "s" : "") +  " verschickt");
+        }
+        catch (Exception re)
+        {
+          Logger.error("", re);
+          monitor.log(re.getMessage());
+        }
+      }
+
+      @Override
+      public void interrupt()
+      {
+        this.cancel = true;
+      }
+
+      @Override
+      public boolean isInterrupted()
+      {
+        return this.cancel;
+      }
+    };
+    Application.getController().start(t);
+  }
+
 }
