@@ -16,8 +16,10 @@
  **********************************************************************/
 package de.jost_net.JVerein.gui.menu;
 
+import de.jost_net.JVerein.gui.action.BuchungsartAction;
 import de.jost_net.JVerein.gui.action.BuchungsartDeleteAction;
 import de.willuhn.jameica.gui.parts.CheckedContextMenuItem;
+import de.willuhn.jameica.gui.parts.CheckedSingleContextMenuItem;
 import de.willuhn.jameica.gui.parts.ContextMenu;
 
 /**
@@ -31,7 +33,9 @@ public class BuchungsartMenu extends ContextMenu
    */
   public BuchungsartMenu()
   {
-    addItem(new CheckedContextMenuItem("lschen...",
-        new BuchungsartDeleteAction(), "trash-alt.png"));
+    addItem(new CheckedSingleContextMenuItem("Bearbeiten", new BuchungsartAction(),
+        "text-x-generic.png"));
+    addItem(new CheckedContextMenuItem("Lschen",
+        new BuchungsartDeleteAction(), "user-trash-full.png"));
   }
 }
