@@ -16,8 +16,10 @@
  **********************************************************************/
 package de.jost_net.JVerein.gui.menu;
 
+import de.jost_net.JVerein.gui.action.BuchungsklasseAction;
 import de.jost_net.JVerein.gui.action.BuchungsklasseDeleteAction;
 import de.willuhn.jameica.gui.parts.CheckedContextMenuItem;
+import de.willuhn.jameica.gui.parts.CheckedSingleContextMenuItem;
 import de.willuhn.jameica.gui.parts.ContextMenu;
 
 /**
@@ -31,7 +33,9 @@ public class BuchungsklasseMenu extends ContextMenu
    */
   public BuchungsklasseMenu()
   {
-    addItem(new CheckedContextMenuItem("lschen...",
-        new BuchungsklasseDeleteAction(), "trash-alt.png"));
+    addItem(new CheckedSingleContextMenuItem("Bearbeiten", new BuchungsklasseAction(),
+        "text-x-generic.png"));
+    addItem(new CheckedContextMenuItem("Lschen",
+        new BuchungsklasseDeleteAction(), "user-trash-full.png"));
   }
 }
