@@ -16,19 +16,22 @@
  **********************************************************************/
 package de.jost_net.JVerein.io;
 
+import java.math.RoundingMode;
 import java.io.File;
 import java.io.IOException;
+import java.io.OutputStream;
 import java.math.BigDecimal;
+import java.nio.file.Files;
 import java.rmi.RemoteException;
+import java.text.DateFormat;
+import java.text.SimpleDateFormat;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Calendar;
 import java.util.Date;
 import java.util.List;
 import java.util.Map;
-
-import javax.xml.bind.JAXBException;
-import javax.xml.datatype.DatatypeConfigurationException;
+import java.util.Properties;
 
 import com.itextpdf.text.DocumentException;
 
@@ -39,7 +42,6 @@ import de.jost_net.JVerein.Variable.MitgliedMap;
 import de.jost_net.JVerein.io.Adressbuch.Adressaufbereitung;
 import de.jost_net.JVerein.keys.Abrechnungsausgabe;
 import de.jost_net.JVerein.keys.Abrechnungsmodi;
-import de.jost_net.JVerein.keys.ArtBeitragsart;
 import de.jost_net.JVerein.keys.Beitragsmodel;
 import de.jost_net.JVerein.keys.IntervallZusatzzahlung;
 import de.jost_net.JVerein.keys.Zahlungsrhythmus;
@@ -70,6 +72,7 @@ import de.jost_net.OBanToo.StringLatin.Zeichen;
 import de.willuhn.datasource.rmi.DBIterator;
 import de.willuhn.jameica.gui.GUI;
 import de.willuhn.jameica.gui.internal.action.Program;
+import de.willuhn.jameica.hbci.HBCIProperties;
 import de.willuhn.jameica.hbci.io.SepaLastschriftMerger;
 import de.willuhn.jameica.hbci.rmi.SepaLastSequenceType;
 import de.willuhn.jameica.hbci.rmi.SepaLastType;
@@ -80,10 +83,14 @@ import de.willuhn.jameica.system.Application;
 import de.willuhn.logging.Logger;
 import de.willuhn.util.ApplicationException;
 import de.willuhn.util.ProgressMonitor;
+import org.apache.commons.lang.StringUtils;
+import org.kapott.hbci.GV.SepaUtil;
+import org.kapott.hbci.GV.generators.ISEPAGenerator;
+import org.kapott.hbci.GV.generators.SEPAGeneratorFactory;
 
 public class AbrechnungSEPA
 {
-  private Calendar sepagueltigkeit;
+  private final Calendar sepagueltigkeit;
 
   private int counter = 0;
 
@@ -111,7 +118,7 @@ public class AbrechnungSEPA
 
     sepagueltigkeit = Calendar.getInstance();
     sepagueltigkeit.add(Calendar.MONTH, -36);
-    JVereinBasislastschrift lastschrift = new JVereinBasislastschrift();
+    Basislastschrift lastschrift = new Basislastschrift();
     // Vorbereitung: Allgemeine Informationen einstellen
     lastschrift.setBIC(Einstellungen.getEinstellung().getBic());
     lastschrift
@@ -131,16 +138,31 @@ public class AbrechnungSEPA
     }
     if (param.kursteilnehmer)
     {
-      abbuchenKursteilnehmer(param, lastschrift);
+      abbuchenKursteilnehmer(param, lastschrift, abrl, konto, monitor);
     }
 
     monitor.log(counter + " abgerechnete Flle");
 
-    lastschrift.setMessageID(abrl.getID());
-    lastschrift.write(param.sepafileFRST, param.sepafileRCUR);
+    lastschrift.setMessageID(abrl.getID() + "-RCUR");
+    if (param.kompakteabbuchung || param.sepaprint)
+    {
+      // Fr kompakte Abbuchung wird erst in write die Zahlerliste gefllt. Das fr die
+      // PDF-Erzeugung bentigte Datum wird auch erst in write gesetzt
+      File temp_file = Files.createTempFile("jv", ".xml").toFile();
+      lastschrift.write(temp_file);
+      temp_file.delete();
+    }
 
     ArrayList<Zahler> z = lastschrift.getZahler();
-    BigDecimal summemitgliedskonto = new BigDecimal("0");
+    // Wenn keine Buchungen vorhanden sind, wird kein File erzeugt.
+    if ((param.abbuchungsausgabe == Abrechnungsausgabe.SEPA_DATEI) && !z.isEmpty())
+    {
+      writeSepaFile(param, lastschrift, z);
+      monitor.log(String.format("SEPA-Datei %s geschrieben.", param.sepafileRCUR.getAbsolutePath()));
+      param.setText(String.format(", SEPA-Datei %s geschrieben.", param.sepafileRCUR.getAbsolutePath()));
+    }
+
+    BigDecimal summemitgliedskonto = BigDecimal.valueOf(0);
     for (Zahler za : z)
     {
       Lastschrift ls = (Lastschrift) Einstellungen.getDBService()
@@ -168,6 +190,8 @@ public class AbrechnungSEPA
           ls.setOrt(k.getOrt());
           ls.setStaat(k.getStaat());
           ls.setEmail(k.getEmail());
+          if (k.getGeschlecht() != null)
+            ls.setGeschlecht(k.getGeschlecht());
           break;
         case MITGLIED:
           ls.setMitglied(Integer.parseInt(vza.getPersonId()));
@@ -219,24 +243,29 @@ public class AbrechnungSEPA
     }
 
     // Gegenbuchung fr das Mitgliedskonto schreiben
-    if (!summemitgliedskonto.equals(new BigDecimal("0")))
+    if (!summemitgliedskonto.equals(BigDecimal.valueOf(0)))
     {
       writeMitgliedskonto(null, new Date(), "Gegenbuchung",
-          summemitgliedskonto.doubleValue() * -1, abrl, true, getKonto(), null);
+          summemitgliedskonto.doubleValue() * -1, abrl, true, getKonto(), null, null);
     }
     if (param.abbuchungsausgabe == Abrechnungsausgabe.HIBISCUS)
     {
-      buchenHibiscus(param, z);
+      // Wenn keine Buchungen vorhanden sind, wird nichts an Hibiscus bergeben.
+      if (z.size() != 0)
+      {
+        buchenHibiscus(param, z);
+        monitor.log("Hibiscus-Lastschrift erzeugt.");
+        param.setText(String.format(", Hibiscus-Lastschrift erzeugt."));
+      }
     }
-    monitor.setPercentComplete(100);
-    if (param.sepaprint)
+    if (param.pdffileRCUR != null)
     {
-      ausdruckenSEPA(lastschrift, param.pdffileFRST, param.pdffileRCUR);
+      ausdruckenSEPA(lastschrift, param.pdffileRCUR);
     }
   }
 
   private void abrechnenMitglieder(AbrechnungSEPAParam param,
-      JVereinBasislastschrift lastschrift, ProgressMonitor monitor,
+      Basislastschrift lastschrift, ProgressMonitor monitor,
       Abrechnungslauf abrl, Konto konto) throws Exception
   {
     if (param.abbuchungsmodus != Abrechnungsmodi.KEINBEITRAG)
@@ -263,7 +292,7 @@ public class AbrechnungSEPA
       // bercksichtigt, die ab einem bestimmten Zeitpunkt eingetreten sind.
       if (param.vondatum != null)
       {
-        list.addFilter("eingabedatum >= ?",
+        list.addFilter("eintritt >= ?",
             new Object[] { new java.sql.Date(param.vondatum.getTime()) });
       }
       if (Einstellungen.getEinstellung()
@@ -273,17 +302,17 @@ public class AbrechnungSEPA
         {
           list.addFilter(
               "(zahlungsrhytmus = ? or zahlungsrhytmus = ? or zahlungsrhytmus = ?)",
-              new Object[] { new Integer(Zahlungsrhythmus.HALBJAEHRLICH),
-                  new Integer(Zahlungsrhythmus.VIERTELJAEHRLICH),
-                  new Integer(Zahlungsrhythmus.MONATLICH) });
+              new Object[] { Integer.valueOf(Zahlungsrhythmus.HALBJAEHRLICH),
+                  Integer.valueOf(Zahlungsrhythmus.VIERTELJAEHRLICH),
+                  Integer.valueOf(Zahlungsrhythmus.MONATLICH) });
         }
         if (param.abbuchungsmodus == Abrechnungsmodi.JAVIMO)
         {
           list.addFilter(
               "(zahlungsrhytmus = ? or zahlungsrhytmus = ? or zahlungsrhytmus = ?)",
-              new Object[] { new Integer(Zahlungsrhythmus.JAEHRLICH),
-                  new Integer(Zahlungsrhythmus.VIERTELJAEHRLICH),
-                  new Integer(Zahlungsrhythmus.MONATLICH) });
+              new Object[] { Integer.valueOf(Zahlungsrhythmus.JAEHRLICH),
+                  Integer.valueOf(Zahlungsrhythmus.VIERTELJAEHRLICH),
+                  Integer.valueOf(Zahlungsrhythmus.MONATLICH) });
         }
         if (param.abbuchungsmodus == Abrechnungsmodi.VIMO)
         {
@@ -313,16 +342,15 @@ public class AbrechnungSEPA
         }
       }
 
-      list.setOrder("ORDER BY name, vorname");
+      list.setOrder("ORDER BY zahlungsweg, name, vorname");
 
       // Stze im Resultset
       int count = 0;
       while (list.hasNext())
       {
-        monitor.setStatus((int) ((double) count / (double) list.size() * 100d));
         Mitglied m = (Mitglied) list.next();
 
-        JVereinZahler z = abrechnungMitgliederSub(param, lastschrift, monitor,
+        JVereinZahler z = abrechnungMitgliederSub(param, monitor,
             abrl, konto, m, m.getBeitragsgruppe(), true);
 
         DBIterator<SekundaereBeitragsgruppe> sekundaer = Einstellungen
@@ -332,8 +360,7 @@ public class AbrechnungSEPA
         {
           SekundaereBeitragsgruppe sb = (SekundaereBeitragsgruppe) sekundaer
               .next();
-          JVereinZahler z2 = abrechnungMitgliederSub(param, lastschrift,
-              monitor, abrl, konto, m, sb.getBeitragsgruppe(), false);
+          JVereinZahler z2 = abrechnungMitgliederSub(param, monitor, abrl, konto, m, sb.getBeitragsgruppe(), false);
           if (z2 != null)
           {
             if (z != null)
@@ -350,22 +377,31 @@ public class AbrechnungSEPA
         {
           lastschrift.add(z);
         }
+        monitor.setPercentComplete((int) ((double) count++ / (double) list.size() * 100d));
+        monitor.setStatusText(String.format("%s, %s abgerechnet" , m.getName(), m.getVorname()));
       }
     }
   }
 
-  private JVereinZahler abrechnungMitgliederSub(AbrechnungSEPAParam param,
-      JVereinBasislastschrift lastschrift, ProgressMonitor monitor,
+  private JVereinZahler abrechnungMitgliederSub(AbrechnungSEPAParam param, ProgressMonitor monitor,
       Abrechnungslauf abrl, Konto konto, Mitglied m, Beitragsgruppe bg,
       boolean primaer) throws RemoteException, ApplicationException
   {
     Double betr = 0d;
     JVereinZahler zahler = null;
+    Mitglied mZahler = m;
+    if(m.getZahlungsweg() != null && m.getZahlungsweg() == Zahlungsweg.VOLLZAHLER)
+    {
+      if(m.getZahlerID() == null)
+        throw new ApplicationException(
+          "Kein Vollzahler vorhanden: " + m.getName() + ", " + m.getVorname());
+      mZahler = Einstellungen.getDBService().createObject(Mitglied.class, m.getZahlerID().toString());
+    }
     if (Einstellungen.getEinstellung()
         .getBeitragsmodel() == Beitragsmodel.FLEXIBEL)
     {
-      if (m.getZahlungstermin() != null
-          && !m.getZahlungstermin().isAbzurechnen(param.abrechnungsmonat))
+      if (mZahler.getZahlungstermin() != null
+          && !mZahler.getZahlungstermin().isAbzurechnen(param.abrechnungsmonat))
       {
         return zahler;
       }
@@ -375,8 +411,8 @@ public class AbrechnungSEPA
     {
       betr = BeitragsUtil.getBeitrag(
           Einstellungen.getEinstellung().getBeitragsmodel(),
-          m.getZahlungstermin(), m.getZahlungsrhythmus().getKey(), bg,
-          param.stichtag, m.getEintritt(), m.getAustritt());
+          mZahler.getZahlungstermin(), mZahler.getZahlungsrhythmus().getKey(), bg,
+          param.stichtag, m);
     }
     catch (NullPointerException e)
     {
@@ -386,7 +422,7 @@ public class AbrechnungSEPA
     if (primaer)
     {
       if (Einstellungen.getEinstellung().getIndividuelleBeitraege()
-          && m.getIndividuellerBeitrag() > 0)
+          && m.getIndividuellerBeitrag() != null)
       {
         betr = m.getIndividuellerBeitrag();
       }
@@ -395,7 +431,7 @@ public class AbrechnungSEPA
     {
       return zahler;
     }
-    if (!checkSEPA(m, monitor))
+    if (!checkSEPA(mZahler, monitor))
     {
       return zahler;
     }
@@ -413,58 +449,43 @@ public class AbrechnungSEPA
     }
 
     writeMitgliedskonto(m,
-        m.getMandatSequence().getTxt().equals("FRST") ? param.faelligkeit1
-            : param.faelligkeit2,
+        param.faelligkeit,
         primaer ? vzweck : bg.getBezeichnung(), betr, abrl,
-        m.getZahlungsweg() == Zahlungsweg.BASISLASTSCHRIFT, konto,
-        bg.getBuchungsart());
-    if (m.getZahlungsweg() == Zahlungsweg.BASISLASTSCHRIFT)
+        mZahler.getZahlungsweg() == Zahlungsweg.BASISLASTSCHRIFT, konto,
+        bg.getBuchungsart(), bg.getBuchungsklasseId());
+    if (mZahler.getZahlungsweg() == Zahlungsweg.BASISLASTSCHRIFT)
     {
       try
       {
         zahler = new JVereinZahler();
-        zahler.setPersonId(m.getID());
+        zahler.setPersonId(mZahler.getID());
         zahler.setPersonTyp(JVereinZahlerTyp.MITGLIED);
         zahler.setBetrag(
-            new BigDecimal(betr).setScale(2, BigDecimal.ROUND_HALF_UP));
-        new BIC(m.getBic()); // Prfung des BIC
-        zahler.setBic(m.getBic());
-        new IBAN(m.getIban()); // Prfung der IBAN
-        zahler.setIban(m.getIban());
-        zahler.setMandatid(m.getMandatID());
-        zahler.setMandatdatum(m.getMandatDatum());
-        zahler.setMandatsequence(m.getMandatSequence());
-        zahler.setFaelligkeit(param.faelligkeit1, param.faelligkeit2,
-            m.getMandatSequence().getCode());
-        if (primaer)
+            BigDecimal.valueOf(betr).setScale(2, RoundingMode.HALF_UP));
+        IBAN i = new IBAN(mZahler.getIban()); // Prfung der IBAN
+        zahler.setIban(mZahler.getIban());
+        //Wenn BIC nicht vorhanden versuchen sie automatisch zu ermitteln
+        if(mZahler.getBic() == null || mZahler.getBic().length() == 0)
+          zahler.setBic(i.getBIC());
+        else zahler.setBic(mZahler.getBic());
+        new BIC(zahler.getBic()); // Prfung des BIC
+        zahler.setMandatid(mZahler.getMandatID());
+        zahler.setMandatdatum(mZahler.getMandatDatum());
+        zahler.setMandatsequence(MandatSequence.RCUR);
+        zahler.setFaelligkeit(param.faelligkeit);
+        if (primaer && m.getZahlungsweg() != Zahlungsweg.VOLLZAHLER)
         {
-          zahler.setVerwendungszweck(getVerwendungszweck2(m) + " " + vzweck);
+          zahler.setVerwendungszweck(getVerwendungszweck2(mZahler) + " " + vzweck);
         }
         else
         {
           zahler.setVerwendungszweck(bg.getBezeichnung());
         }
-        if (m.getBeitragsgruppe()
-            .getBeitragsArt() == ArtBeitragsart.FAMILIE_ZAHLER)
+        if(m.getZahlungsweg() == Zahlungsweg.VOLLZAHLER)
         {
-          DBIterator<Mitglied> angeh = Einstellungen.getDBService()
-              .createList(Mitglied.class);
-          angeh.addFilter("zahlerid = ?", m.getID());
-          String an = "";
-          int i = 0;
-          while (angeh.hasNext())
-          {
-            Mitglied a = (Mitglied) angeh.next();
-            if (i > 0)
-            {
-              an += ", ";
-            }
-            i++;
-            an += a.getVorname();
-          }
-          zahler.setVerwendungszweck(zahler.getVerwendungszweck() + " " + an);
+          zahler.setVerwendungszweck(zahler.getVerwendungszweck() + " " + m.getVorname());
         }
-        zahler.setName(m.getKontoinhaber(1));
+        zahler.setName(mZahler.getKontoinhaber(1));
       }
       catch (Exception e)
       {
@@ -476,10 +497,11 @@ public class AbrechnungSEPA
   }
 
   private void abbuchenZusatzbetraege(AbrechnungSEPAParam param,
-      JVereinBasislastschrift lastschrift, Abrechnungslauf abrl, Konto konto,
+      Basislastschrift lastschrift, Abrechnungslauf abrl, Konto konto,
       ProgressMonitor monitor)
       throws NumberFormatException, IOException, ApplicationException
   {
+    int count = 0;
     DBIterator<Zusatzbetrag> list = Einstellungen.getDBService()
         .createList(Zusatzbetrag.class);
     while (list.hasNext())
@@ -497,7 +519,12 @@ public class AbrechnungSEPA
         {
           continue;
         }
-        if (!checkSEPA(m, monitor))
+        Mitglied mZahler = m;
+        if(m.getZahlungsweg() != null && m.getZahlungsweg() == Zahlungsweg.VOLLZAHLER)
+        {
+          mZahler = Einstellungen.getDBService().createObject(Mitglied.class, m.getZahlerID().toString());
+        }
+        if (!checkSEPA(mZahler, monitor))
         {
           continue;
         }
@@ -514,25 +541,24 @@ public class AbrechnungSEPA
         {
           Logger.error("Fehler bei der Aufbereitung der Variablen", e);
         }
-        if (m.getZahlungsweg() == Zahlungsweg.BASISLASTSCHRIFT)
+        if (mZahler.getZahlungsweg() == Zahlungsweg.BASISLASTSCHRIFT)
         {
           try
           {
             JVereinZahler zahler = new JVereinZahler();
-            zahler.setPersonId(m.getID());
+            zahler.setPersonId(mZahler.getID());
             zahler.setPersonTyp(JVereinZahlerTyp.MITGLIED);
-            zahler.setBetrag(new BigDecimal(z.getBetrag()).setScale(2,
-                BigDecimal.ROUND_HALF_UP));
-            new BIC(m.getBic());
-            new IBAN(m.getIban());
-            zahler.setBic(m.getBic());
-            zahler.setIban(m.getIban());
-            zahler.setMandatid(m.getMandatID());
-            zahler.setMandatdatum(m.getMandatDatum());
-            zahler.setMandatsequence(m.getMandatSequence());
-            zahler.setFaelligkeit(param.faelligkeit1, param.faelligkeit2,
-                m.getMandatSequence().getCode());
-            zahler.setName(m.getKontoinhaber(1));
+            zahler.setBetrag(BigDecimal.valueOf(z.getBetrag()).setScale(2,
+                RoundingMode.HALF_UP));
+            new BIC(mZahler.getBic());
+            new IBAN(mZahler.getIban());
+            zahler.setBic(mZahler.getBic());
+            zahler.setIban(mZahler.getIban());
+            zahler.setMandatid(mZahler.getMandatID());
+            zahler.setMandatdatum(mZahler.getMandatDatum());
+            zahler.setMandatsequence(MandatSequence.RCUR);
+            zahler.setFaelligkeit(param.faelligkeit);
+            zahler.setName(mZahler.getKontoinhaber(1));
             zahler.setVerwendungszweck(vzweck);
             lastschrift.add(zahler);
           }
@@ -575,19 +601,22 @@ public class AbrechnungSEPA
           throw e;
         }
         writeMitgliedskonto(m,
-            m.getMandatSequence().getTxt().equals("FRST") ? param.faelligkeit1
-                : param.faelligkeit2,
+            param.faelligkeit,
             vzweck, z.getBetrag(), abrl,
-            m.getZahlungsweg() == Zahlungsweg.BASISLASTSCHRIFT, konto,
-            z.getBuchungsart());
+            mZahler.getZahlungsweg() == Zahlungsweg.BASISLASTSCHRIFT, konto,
+            z.getBuchungsart(),
+            z.getBuchungsklasseId());
+        monitor.setStatusText(String.format("Zusatzbetrag von %s, %s abgerechnet" , m.getName(), m.getVorname()));
       }
+      monitor.setPercentComplete((int) ((double) count++ / (double) list.size() * 100d));
     }
   }
 
   private void abbuchenKursteilnehmer(AbrechnungSEPAParam param,
-      JVereinBasislastschrift lastschrift)
+      Basislastschrift lastschrift, Abrechnungslauf abrl, Konto konto, ProgressMonitor monitor)
       throws ApplicationException, IOException
   {
+    int count = 0;
     DBIterator<Kursteilnehmer> list = Einstellungen.getDBService()
         .createList(Kursteilnehmer.class);
     list.addFilter("abbudatum is null");
@@ -600,21 +629,25 @@ public class AbrechnungSEPA
         JVereinZahler zahler = new JVereinZahler();
         zahler.setPersonId(kt.getID());
         zahler.setPersonTyp(JVereinZahlerTyp.KURSTEILNEHMER);
-        zahler.setBetrag(new BigDecimal(kt.getBetrag()).setScale(2,
-            BigDecimal.ROUND_HALF_UP));
+        zahler.setBetrag(BigDecimal.valueOf(kt.getBetrag()).setScale(2,
+            RoundingMode.HALF_UP));
         new BIC(kt.getBic());
         new IBAN(kt.getIban());
         zahler.setBic(kt.getBic());
         zahler.setIban(kt.getIban());
         zahler.setMandatid(kt.getMandatID());
         zahler.setMandatdatum(kt.getMandatDatum());
-        zahler.setMandatsequence(MandatSequence.FRST);
-        zahler.setFaelligkeit(param.faelligkeit1);
+        zahler.setMandatsequence(MandatSequence.RCUR);
+        zahler.setFaelligkeit(param.faelligkeit);
         zahler.setName(kt.getName());
         zahler.setVerwendungszweck(kt.getVZweck1());
         lastschrift.add(zahler);
-        kt.setAbbudatum();
+        kt.setAbbudatum(param.faelligkeit);
         kt.store();
+        writeMitgliedskonto(kt, param.faelligkeit, kt.getVZweck1(), 
+            zahler.getBetrag().doubleValue(), abrl, true, konto, null, null);
+        monitor.setStatusText(String.format("Kursteilnehmer %s, %s abgerechnet" , kt.getName(), kt.getVorname()));
+        monitor.setPercentComplete((int) ((double) count++ / (double) list.size() * 100d));
       }
       catch (Exception e)
       {
@@ -623,11 +656,10 @@ public class AbrechnungSEPA
     }
   }
 
-  private void ausdruckenSEPA(final JVereinBasislastschrift lastschrift,
-      final String pdfFRST, final String pdfRCUR)
+  private void ausdruckenSEPA(final Basislastschrift lastschrift, final String pdf_fn)
       throws IOException, DocumentException, SEPAException
   {
-    new Basislastschrift2Pdf(lastschrift.getLastschriftFRST(), pdfFRST);
+    new Basislastschrift2Pdf(lastschrift, pdf_fn);
     GUI.getDisplay().asyncExec(new Runnable()
     {
 
@@ -636,25 +668,7 @@ public class AbrechnungSEPA
       {
         try
         {
-          new Program().handleAction(new File(pdfFRST));
-        }
-        catch (ApplicationException ae)
-        {
-          Application.getMessagingFactory().sendMessage(new StatusBarMessage(
-              ae.getLocalizedMessage(), StatusBarMessage.TYPE_ERROR));
-        }
-      }
-    });
-    new Basislastschrift2Pdf(lastschrift.getLastschriftRCUR(), pdfRCUR);
-    GUI.getDisplay().asyncExec(new Runnable()
-    {
-
-      @Override
-      public void run()
-      {
-        try
-        {
-          new Program().handleAction(new File(pdfRCUR));
+          new Program().handleAction(new File(pdf_fn));
         }
         catch (ApplicationException ae)
         {
@@ -665,14 +679,48 @@ public class AbrechnungSEPA
     });
   }
 
+  private void writeSepaFile(AbrechnungSEPAParam param, Basislastschrift lastschrift, ArrayList<Zahler> alle_zahler) throws Exception
+  {
+    Properties ls_properties = new Properties();
+    ls_properties.setProperty("src.bic", lastschrift.getBIC());
+    ls_properties.setProperty("src.iban", lastschrift.getIBAN());
+    ls_properties.setProperty("src.name", lastschrift.getName());
+    long epochtime = Calendar.getInstance().getTimeInMillis();
+    String epochtime_string = Long.toString(epochtime);
+    DateFormat ISO_DATE = new SimpleDateFormat(SepaUtil.DATE_FORMAT);
+    ls_properties.setProperty("sepaid", epochtime_string);
+    ls_properties.setProperty("pmtinfid", epochtime_string);
+    ls_properties.setProperty("sequencetype", "RCUR");
+    ls_properties.setProperty("targetdate", param.faelligkeit != null ? ISO_DATE.format(param.faelligkeit) : SepaUtil.DATE_UNDEFINED);
+    ls_properties.setProperty("type", "CORE");
+    ls_properties.setProperty("batchbook", "");
+    int counter = 0;
+    String creditorid = lastschrift.getGlaeubigerID();
+    for (Zahler zahler : alle_zahler)
+    {
+      ls_properties.setProperty(SepaUtil.insertIndex("dst.bic", counter),       StringUtils.trimToEmpty(zahler.getBic()));
+      ls_properties.setProperty(SepaUtil.insertIndex("dst.iban", counter),      StringUtils.trimToEmpty(zahler.getIban()));
+      ls_properties.setProperty(SepaUtil.insertIndex("dst.name", counter),      StringUtils.trimToEmpty(zahler.getName()));
+      ls_properties.setProperty(SepaUtil.insertIndex("btg.value", counter),     zahler.getBetrag().toString());
+      ls_properties.setProperty(SepaUtil.insertIndex("btg.curr", counter),      HBCIProperties.CURRENCY_DEFAULT_DE);
+      ls_properties.setProperty(SepaUtil.insertIndex("usage", counter),         StringUtils.trimToEmpty(zahler.getVerwendungszweck()));
+      ls_properties.setProperty(SepaUtil.insertIndex("endtoendid", counter),    "NOTPROVIDED");
+      ls_properties.setProperty(SepaUtil.insertIndex("creditorid", counter),    creditorid);
+      ls_properties.setProperty(SepaUtil.insertIndex("mandateid", counter),     StringUtils.trimToEmpty(zahler.getMandatid()));
+      ls_properties.setProperty(SepaUtil.insertIndex("manddateofsig", counter), ISO_DATE.format(zahler.getMandatdatum()));
+      ls_properties.setProperty(SepaUtil.insertIndex("purposecode", counter),   "OHTR");
+      counter += 1;
+    }
+    final OutputStream os = Files.newOutputStream(param.sepafileRCUR.toPath());
+    System.setProperty("sepa.pain.formatted", "true");
+    ISEPAGenerator sepagenerator = SEPAGeneratorFactory.get("LastSEPA", param.sepaVersion);
+    sepagenerator.generate(ls_properties, os, true);
+    os.close();
+  }
+
   private void buchenHibiscus(AbrechnungSEPAParam param, ArrayList<Zahler> z)
       throws ApplicationException
   {
-    if (z.size() == 0)
-    {
-      // Wenn keine Buchungen vorhanden sind, wird nichts an Hibiscus bergeben.
-      return;
-    }
     try
     {
       SepaLastschrift[] lastschriften = new SepaLastschrift[z.size()];
@@ -745,8 +793,7 @@ public class AbrechnungSEPA
         .createObject(Abrechnungslauf.class, null);
     abrl.setDatum(new Date());
     abrl.setAbbuchungsausgabe(param.abbuchungsausgabe.getKey());
-    abrl.setFaelligkeit(param.faelligkeit1);
-    abrl.setFaelligkeit2(param.faelligkeit2);
+    abrl.setFaelligkeit(param.faelligkeit);
     abrl.setDtausdruck(param.sepaprint);
     abrl.setEingabedatum(param.vondatum);
     abrl.setAustrittsdatum(param.bisdatum);
@@ -760,28 +807,39 @@ public class AbrechnungSEPA
     return abrl;
   }
 
-  private void writeMitgliedskonto(Mitglied mitglied, Date datum, String zweck1,
+  private void writeMitgliedskonto(Object mitglied, Date datum, String zweck1,
       double betrag, Abrechnungslauf abrl, boolean haben, Konto konto,
-      Buchungsart buchungsart) throws ApplicationException, RemoteException
+      Buchungsart buchungsart, Long buchungsklasseId) throws ApplicationException, RemoteException
   {
     Mitgliedskonto mk = null;
-    if (mitglied != null) /*
+    if (mitglied != null && mitglied instanceof Mitglied) /*
                            * Mitglied darf dann null sein, wenn die Gegenbuchung
                            * geschrieben wird
                            */
     {
+      Mitglied mg = (Mitglied) mitglied;
       mk = (Mitgliedskonto) Einstellungen.getDBService()
           .createObject(Mitgliedskonto.class, null);
       mk.setAbrechnungslauf(abrl);
-      mk.setZahlungsweg(mitglied.getZahlungsweg());
+      mk.setZahlungsweg(mg.getZahlungsweg());
       mk.setBetrag(betrag);
       mk.setDatum(datum);
-      mk.setMitglied(mitglied);
+      mk.setMitglied(mg);
       mk.setZweck1(zweck1);
+      double steuersatz = 0d;
       if (buchungsart != null)
       {
         mk.setBuchungsart(buchungsart);
+        steuersatz = buchungsart.getSteuersatz();
       }
+      mk.setBuchungsklasseId(buchungsklasseId);
+      // Set tax rate
+      mk.setSteuersatz(steuersatz);
+      // Set bill amount without taxes
+      double nettobetrag = (steuersatz != 0d) ? (betrag / (1d + (steuersatz / 100d))) : betrag;
+      mk.setNettobetrag(nettobetrag);
+      // Set tax amount
+      mk.setSteuerbetrag(betrag - nettobetrag);    
       mk.store();
     }
     if (haben)
@@ -793,7 +851,7 @@ public class AbrechnungSEPA
       buchung.setDatum(datum);
       buchung.setKonto(konto);
       buchung.setName(
-          mitglied != null ? Adressaufbereitung.getNameVorname(mitglied)
+          mitglied != null ? Adressaufbereitung.getNameVorname((IAdresse) mitglied)
               : "JVerein");
       buchung.setZweck(zweck1);
       if (mk != null)
@@ -802,8 +860,9 @@ public class AbrechnungSEPA
       }
       if (buchungsart != null)
       {
-        buchung.setBuchungsart(new Long(buchungsart.getID()));
+        buchung.setBuchungsart(Long.valueOf(buchungsart.getID()));
       }
+      buchung.setBuchungsklasseId(buchungsklasseId);
       buchung.store();
     }
   }
@@ -846,29 +905,21 @@ public class AbrechnungSEPA
   }
 
   private boolean checkSEPA(Mitglied m, ProgressMonitor monitor)
-      throws RemoteException, ApplicationException
+      throws RemoteException
   {
     if (m.getZahlungsweg() == null
         || m.getZahlungsweg() != Zahlungsweg.BASISLASTSCHRIFT)
     {
       return true;
     }
-    if (m.getLetzteLastschrift() != null
-        && m.getLetzteLastschrift().before(sepagueltigkeit.getTime()))
+    Date letzte_lastschrift = m.getLetzteLastschrift();
+    if (letzte_lastschrift != null
+        && letzte_lastschrift.before(sepagueltigkeit.getTime()))
     {
       monitor.log(Adressaufbereitung.getNameVorname(m)
           + ": Letzte Lastschrift ist lter als 36 Monate.");
       return false;
     }
-    if (m.getMandatSequence().equals(MandatSequence.FRST)
-        && m.getLetzteLastschrift() != null)
-    {
-      Mitglied m1 = (Mitglied) Einstellungen.getDBService()
-          .createObject(Mitglied.class, m.getID());
-      m1.setMandatSequence(MandatSequence.RCUR);
-      m1.store();
-      m.setMandatSequence(MandatSequence.RCUR);
-    }
     if (m.getMandatDatum() == Einstellungen.NODATE)
     {
       monitor.log(Adressaufbereitung.getNameVorname(m)
@@ -879,99 +930,3 @@ public class AbrechnungSEPA
   }
 
 }
-
-class JVereinBasislastschrift
-{
-  private Basislastschrift lastschriftFRST;
-
-  private Basislastschrift lastschriftRCUR;
-
-  public JVereinBasislastschrift()
-  {
-    lastschriftFRST = new Basislastschrift();
-    lastschriftRCUR = new Basislastschrift();
-  }
-
-  public void setBIC(String bic) throws SEPAException
-  {
-    lastschriftFRST.setBIC(bic);
-    lastschriftRCUR.setBIC(bic);
-  }
-
-  public void setGlaeubigerID(String glaeubigerid)
-      throws RemoteException, SEPAException
-  {
-    lastschriftFRST.setGlaeubigerID(glaeubigerid);
-    lastschriftRCUR.setGlaeubigerID(glaeubigerid);
-  }
-
-  public void setIBAN(String iban) throws SEPAException
-  {
-    lastschriftFRST.setIBAN(iban);
-    lastschriftRCUR.setIBAN(iban);
-  }
-
-  public void setKomprimiert(boolean kompakt) throws SEPAException
-  {
-    lastschriftFRST.setKomprimiert(kompakt);
-    lastschriftRCUR.setKomprimiert(kompakt);
-  }
-
-  public void setName(String name) throws SEPAException
-  {
-    lastschriftFRST.setName(name);
-    lastschriftRCUR.setName(name);
-  }
-
-  public void setMessageID(String id) throws SEPAException
-  {
-    lastschriftFRST.setMessageID(id + "-FRST");
-    lastschriftRCUR.setMessageID(id + "-RCUR");
-  }
-
-  public void write(File frst, File rcur)
-      throws DatatypeConfigurationException, SEPAException, JAXBException
-  {
-    lastschriftFRST.write(frst);
-    lastschriftRCUR.write(rcur);
-  }
-
-  public void add(Zahler zahler) throws SEPAException
-  {
-    if (zahler.getMandatsequence().equals(MandatSequence.FRST))
-    {
-      lastschriftFRST.add(zahler);
-    }
-    else if (zahler.getMandatsequence().equals(MandatSequence.RCUR))
-    {
-      lastschriftRCUR.add(zahler);
-    }
-    else
-      throw new SEPAException("Ungltige Sequenz");
-  }
-
-  public Basislastschrift getLastschriftFRST()
-  {
-    return lastschriftFRST;
-  }
-
-  public Basislastschrift getLastschriftRCUR()
-  {
-    return lastschriftRCUR;
-  }
-
-  public ArrayList<Zahler> getZahler()
-  {
-    ArrayList<Zahler> ret = new ArrayList<>();
-    for (Zahler z : lastschriftFRST.getZahler())
-    {
-      ret.add(z);
-    }
-    for (Zahler z : lastschriftRCUR.getZahler())
-    {
-      ret.add(z);
-    }
-    return ret;
-  }
-
-}
