@@ -18,20 +18,26 @@ package de.jost_net.JVerein.server;
 
 import java.io.Serializable;
 import java.rmi.RemoteException;
+import java.text.ParseException;
+import java.util.ArrayList;
 import java.util.Date;
 
+import de.jost_net.JVerein.util.Datum;
+import de.jost_net.JVerein.util.VonBis;
+import de.jost_net.OBanToo.SEPA.IBAN;
+import de.jost_net.OBanToo.SEPA.SEPAException;
+
+import org.kapott.hbci.sepa.SepaVersion;
+
 import de.jost_net.JVerein.Einstellungen;
 import de.jost_net.JVerein.JVereinPlugin;
 import de.jost_net.JVerein.io.AltersgruppenParser;
 import de.jost_net.JVerein.io.JubilaeenParser;
 import de.jost_net.JVerein.keys.Altermodel;
 import de.jost_net.JVerein.keys.Beitragsmodel;
-import de.jost_net.JVerein.keys.BuchungBuchungsartAuswahl;
+import de.jost_net.JVerein.keys.AbstractInputAuswahl;
 import de.jost_net.JVerein.rmi.Einstellung;
 import de.jost_net.JVerein.rmi.Felddefinition;
-import de.jost_net.OBanToo.SEPA.BIC;
-import de.jost_net.OBanToo.SEPA.IBAN;
-import de.jost_net.OBanToo.SEPA.SEPAException;
 import de.willuhn.datasource.db.AbstractDBObject;
 import de.willuhn.datasource.rmi.DBIterator;
 import de.willuhn.datasource.rmi.DBService;
@@ -114,31 +120,7 @@ public class EinstellungImpl extends AbstractDBObject implements Einstellung
       {
         throw new ApplicationException("Bitte Namen eingeben");
       }
-      if (getIban() == null || getIban().length() == 0)
-      {
-        throw new ApplicationException("Bitte IBAN eingeben");
-      }
-      if (getBic() == null || getBic().length() == 0)
-      {
-        throw new ApplicationException("Bitte BIC eingeben");
-      }
-
-      try
-      {
-        new IBAN(getIban());
-      }
-      catch (SEPAException e1)
-      {
-        throw new ApplicationException(e1.getMessage());
-      }
-      try
-      {
-        new BIC(getBic());
-      }
-      catch (SEPAException e1)
-      {
-        throw new ApplicationException(e1.getMessage());
-      }
+      
       try
       {
         new AltersgruppenParser(getAltersgruppen());
@@ -148,6 +130,59 @@ public class EinstellungImpl extends AbstractDBObject implements Einstellung
         throw new ApplicationException(e.getMessage());
       }
 
+      try
+      {
+        AltersgruppenParser ap = new AltersgruppenParser(getBeitragAltersstufen());
+        ArrayList<VonBis> vbs = new ArrayList<VonBis>();
+        while(ap.hasNext())
+        {
+          vbs.add(ap.getNext());
+        }
+        for(int i=0;i<100;i++)
+        {
+          boolean found = false;
+          for(VonBis vb : vbs)
+          {
+            if(i >= vb.getVon() && i <= vb.getBis())
+            {
+              if(found == true)
+                throw new ApplicationException(i + " Jahre ist in mehreren Altersstufen enthalten");
+              found = true;
+            }
+          }
+          if(!found)
+            throw new ApplicationException("Keine passende Altersstufe gefunden fr " + i + " Jahre");
+        }
+      }
+      catch (RuntimeException e)
+      {
+        throw new ApplicationException(e.getMessage().replace("\n", " "));
+      }
+
+      if(getBeginnGeschaeftsjahr() != null)
+      {
+        try
+        {
+          Datum.toDate(getBeginnGeschaeftsjahr()+ 2000);
+        }
+        catch (ParseException e)
+        {
+          throw new ApplicationException("Ungltiges Datumsformat: " + getBeginnGeschaeftsjahr());
+        }
+      }
+
+      if(getIban() != null)
+      {
+        try
+        {
+          new IBAN(getIban());
+        }
+        catch (SEPAException e)
+        {
+          throw new ApplicationException(e.getMessage());
+        }
+      }
+
       if (getDokumentenspeicherung())
       {
         if (!JVereinPlugin.isArchiveServiceActive())
@@ -159,6 +194,7 @@ public class EinstellungImpl extends AbstractDBObject implements Einstellung
       try
       {
         new JubilaeenParser(getJubilaeen());
+        new JubilaeenParser(getAltersjubilaeen());
       }
       catch (RuntimeException e)
       {
@@ -601,19 +637,6 @@ public class EinstellungImpl extends AbstractDBObject implements Einstellung
     setAttribute("mitgliedfoto", Boolean.valueOf(mitgliedfoto));
   }
 
-  // TODO fr Versionsbau deaktivert.
-  // @Override
-  // public Boolean getInventar() throws RemoteException
-  // {
-  // return Util.getBoolean(getAttribute("inventar"));
-  // }
-
-  // @Override
-  // public void setInventar(Boolean inventar) throws RemoteException
-  // {
-  // setAttribute("inventar", Boolean.valueOf(inventar));
-  // }
-
   @Override
   public boolean getUseLesefelder() throws RemoteException
   {
@@ -743,6 +766,34 @@ public class EinstellungImpl extends AbstractDBObject implements Einstellung
   {
     setAttribute("rechnungtextbar", rechnungtextbar);
   }
+  
+  @Override
+  public int getZaehlerLaenge() throws RemoteException 
+  {
+    try {
+      return (int) getAttribute("zaehlerlaenge");
+    } catch (NullPointerException e) {
+      return 5;
+    }
+  }
+
+  @Override
+  public void setZaehlerLaenge(int length) throws RemoteException 
+  {
+    setAttribute("zaehlerlaenge", length);
+  }
+
+  @Override
+  public Boolean getOptiert() throws RemoteException
+  {
+    return Util.getBoolean(getAttribute("optiert"));
+  }
+
+  @Override
+  public void setOptiert(Boolean optiert) throws RemoteException
+  {
+    setAttribute("optiert", optiert);
+  }
 
   @Override
   public Boolean getExterneMitgliedsnummer() throws RemoteException
@@ -831,7 +882,7 @@ public class EinstellungImpl extends AbstractDBObject implements Einstellung
     Double d = (Double) getAttribute("spendenbescheinigungminbetrag");
     if (d == null)
     {
-      d = new Double(0);
+      d = Double.valueOf(0.0d);
     }
     return (d);
   }
@@ -901,12 +952,7 @@ public class EinstellungImpl extends AbstractDBObject implements Einstellung
   @Override
   public String getSmtpPort() throws RemoteException
   {
-    String ret = (String) getAttribute("smtp_port");
-    if (ret == null)
-    {
-      ret = "25";
-    }
-    return ret;
+    return (String) getAttribute("smtp_port");
   }
 
   @Override
@@ -1134,7 +1180,6 @@ public class EinstellungImpl extends AbstractDBObject implements Einstellung
   public void setImapPort(String imap_port) throws RemoteException
   {
     setAttribute("imap_port", imap_port);
-
   }
 
   @Override
@@ -1332,7 +1377,7 @@ public class EinstellungImpl extends AbstractDBObject implements Einstellung
     {
       DBIterator<Felddefinition> it = Einstellungen.getDBService()
           .createList(Felddefinition.class);
-      hasZus = new Boolean(it.size() > 0);
+      hasZus = Boolean.valueOf(it.size() > 0);
     }
     return hasZus;
   }
@@ -1640,6 +1685,19 @@ public class EinstellungImpl extends AbstractDBObject implements Einstellung
     setAttribute("autobuchunguebernahme", autobuchunguebernahme);
   }
 
+  @Override
+  public Boolean getAutomatischeBuchungskorrekturHibiscus() throws RemoteException
+  {
+    return Util.getBoolean(getAttribute("autobuchungskorrekturhibiscus"));
+  }
+
+  @Override
+  public void setAutomatischeBuchungskorrekturHibiscus(Boolean autobuchungskorrekturhibiscus)
+      throws RemoteException
+  {
+    setAttribute("autobuchungskorrekturhibiscus", autobuchungskorrekturhibiscus);
+  }
+
   @Override
   public Boolean getUnterdrueckungOhneBuchung() throws RemoteException
   {
@@ -1652,6 +1710,52 @@ public class EinstellungImpl extends AbstractDBObject implements Einstellung
   {
     setAttribute("unterdrueckungohnebuchung", unterdrueckungohnebuchung);
   }
+    
+  @Override
+  public int getUnterdrueckungLaenge() throws RemoteException 
+  {
+    try {
+      return (int) getAttribute("unterdrueckunglaenge");
+    } catch (NullPointerException e) {
+      return 0;
+    }
+  }
+
+  @Override
+  public void setUnterdrueckungLaenge(int unterdrueckunglaenge) throws RemoteException 
+  {
+    setAttribute("unterdrueckunglaenge", unterdrueckunglaenge);
+  }
+  
+  @Override
+  public int getUnterdrueckungKonten() throws RemoteException 
+  {
+    try {
+      return (int) getAttribute("unterdrueckungkonten");
+    } catch (NullPointerException e) {
+      return 2;
+    }
+  }
+
+  @Override
+  public void setUnterdrueckungKonten(int unterdrueckungkonten) throws RemoteException 
+  {
+    setAttribute("unterdrueckungkonten", unterdrueckungkonten);
+  }
+
+
+  @Override
+  public Boolean getKontonummerInBuchungsliste() throws RemoteException
+  {
+    return Util.getBoolean(getAttribute("kontonummer_in_buchungsliste"));
+  }
+
+  @Override
+  public void setKontonummerInBuchungsliste(Boolean kontonummer_in_buchungsliste)
+      throws RemoteException
+  {
+    setAttribute("kontonummer_in_buchungsliste", kontonummer_in_buchungsliste);
+  }
 
   @Override
   public Integer getSEPADatumOffset() throws RemoteException
@@ -1659,7 +1763,7 @@ public class EinstellungImpl extends AbstractDBObject implements Einstellung
     Integer offset = (Integer) getAttribute("sepadatumoffset");
     if (offset == null)
     {
-      offset = new Integer(0);
+      offset = Integer.valueOf(0);
     }
     return offset;
   }
@@ -1675,7 +1779,7 @@ public class EinstellungImpl extends AbstractDBObject implements Einstellung
   {
     Integer wert = (Integer) getAttribute(COL_BUCHUNG_BUCHUNGSART_AUSWAHL);
     if (null == wert)
-      return BuchungBuchungsartAuswahl.SearchInput;
+      return AbstractInputAuswahl.SearchInput;
     return wert.intValue();
   }
 
@@ -1684,6 +1788,21 @@ public class EinstellungImpl extends AbstractDBObject implements Einstellung
   {
     setAttribute(COL_BUCHUNG_BUCHUNGSART_AUSWAHL, auswahl);
   }
+  
+  @Override
+  public int getMitgliedAuswahl() throws RemoteException
+  {
+    Integer wert = (Integer) getAttribute("mitgliedauswahl");
+    if (null == wert)
+      return AbstractInputAuswahl.SearchInput;
+    return wert.intValue();
+  }
+
+  @Override
+  public void setMitgliedAuswahl(int mitgliedauswahl) throws RemoteException
+  {
+    setAttribute("mitgliedauswahl", mitgliedauswahl);
+  }
 
   @Override
   public int getBuchungsartSort() throws RemoteException
@@ -1713,5 +1832,323 @@ public class EinstellungImpl extends AbstractDBObject implements Einstellung
   {
     setAttribute("abrlabschliessen", Boolean.valueOf(abschliessen));
   }
+  
+  @Override
+  public SepaVersion getSepaVersion() throws RemoteException
+  {
+    String sepaversion = (String) getAttribute("sepaversion");
+    if (sepaversion == null || !(sepaversion.length() > 0))
+    {
+      return null;
+    }
+    return  SepaVersion.byURN(sepaversion);
+  }
 
+  @Override
+  public void setSepaVersion(SepaVersion sepaversion) throws RemoteException
+  {
+    if (sepaversion == null)
+    {
+      setAttribute("sepaversion", null);
+    }
+    else
+    {
+      setAttribute("sepaversion", sepaversion.getFile());
+    }
+  }
+  
+  @Override
+  public SepaVersion getCt1SepaVersion() throws RemoteException
+  {
+    String ct1sepaversion = (String) getAttribute("ct1sepaversion");
+    if (ct1sepaversion == null || !(ct1sepaversion.length() > 0))
+    {
+      return null;
+    }
+    return  SepaVersion.byURN(ct1sepaversion);
+  }
+
+  @Override
+  public void setCt1SepaVersion(SepaVersion ct1sepaversion) throws RemoteException
+  {
+    if (ct1sepaversion == null)
+    {
+      setAttribute("ct1sepaversion", null);
+    }
+    else
+    {
+      setAttribute("ct1sepaversion", ct1sepaversion.getFile());
+    }
+  }
+
+  @Override
+  public int getQRCodeSizeInMm() throws RemoteException
+  {
+    Integer qrsizemm = (Integer) getAttribute("qrcodesizemm");
+    if (null == qrsizemm)
+    {
+      return 20;
+    }
+    return qrsizemm.intValue();
+  }
+
+  @Override
+  public void setQRCodeSizeInMm(int size) throws RemoteException
+  {
+    setAttribute("qrcodesizemm", size);
+  }
+
+  @Override
+  public boolean getQRCodeFesterText() throws RemoteException
+  {
+    Boolean printFesterText = (Boolean) getAttribute("qrcodeptext");
+    if (null == printFesterText)
+    {
+      return true;
+    }
+    return printFesterText.booleanValue();
+  }
+
+  @Override
+  public void setQRCodeFesterText(boolean printText) throws RemoteException
+  {
+    setAttribute("qrcodeptext", printText);
+  }
+
+  @Override
+  public boolean getQRCodeDatum() throws RemoteException
+  {
+    Boolean printDatum = (Boolean) getAttribute("qrcodepdate");
+    if (null == printDatum)
+    {
+      return false;
+    }
+    return printDatum.booleanValue();
+  }
+
+  @Override
+  public void setQRCodeDatum(boolean printDate) throws RemoteException
+  {
+    setAttribute("qrcodepdate", printDate);
+  }
+
+  @Override
+  public boolean getQRCodeReNu() throws RemoteException
+  {
+    Boolean printReNr = (Boolean) getAttribute("qrcodeprenum");
+    if (null == printReNr)
+    {
+      return false;
+    }
+    return printReNr.booleanValue();
+  }
+
+  @Override
+  public void setQRCodeReNu(boolean printReNum) throws RemoteException
+  {
+    setAttribute("qrcodeprenum", printReNum);
+  }
+
+  @Override
+  public boolean getQRCodeMember() throws RemoteException
+  {
+    Boolean printMemberNr = (Boolean) getAttribute("qrcodepmnum");
+    if (null == printMemberNr)
+    {
+      return false;
+    }
+    return printMemberNr.booleanValue();
+  }
+
+  @Override
+  public void setQRCodeMember(boolean printMember) throws RemoteException
+  {
+    setAttribute("qrcodepmnum", printMember);
+  }
+
+  @Override
+  public String getQRCodeText() throws RemoteException
+  {
+    String qrText = (String) getAttribute("qrcodetext");
+    if (null == qrText)
+    {
+      return "Mitgliedsbeitrag";
+    }
+    return qrText;
+  }
+
+  @Override
+  public void setQRCodeText(String text) throws RemoteException
+  {
+    setAttribute("qrcodetext", text);
+  }
+
+  @Override
+  public boolean getQRCodeSnglLine() throws RemoteException
+  {
+    Boolean replaceSnglLine = (Boolean) getAttribute("qrcodesngl");
+    if (null == replaceSnglLine)
+    {
+      return true;
+    }
+    return replaceSnglLine.booleanValue();
+  }
+
+  @Override
+  public void setQRCodeSnglLine(boolean single) throws RemoteException
+  {
+    setAttribute("qrcodesngl", single);
+  }
+
+  @Override
+  public String getQRCodeInfoM() throws RemoteException
+  {
+    String qrInfoToMember = (String) getAttribute("qrcodeinfom");
+    if (null == qrInfoToMember)
+    {
+      return "Vielen Dank!";
+    }
+    return qrInfoToMember;
+  }
+
+  @Override
+  public void setQRCodeInfoM(String infoMitglied) throws RemoteException
+  {
+    setAttribute("qrcodeinfom", infoMitglied);
+  }
+
+  @Override
+  public boolean getQRCodeKuerzen() throws RemoteException
+  {
+    Boolean kuerzen = (Boolean) getAttribute("qrcodekuerzen");
+    if (null == kuerzen)
+    {
+      return false;
+    }
+    return kuerzen.booleanValue();
+  }
+
+  @Override
+  public void setQRCodeKuerzen(boolean kuerzen) throws RemoteException
+  {
+    setAttribute("qrcodekuerzen", kuerzen);
+  }
+
+  @Override
+  public String getQRCodeIntro() throws RemoteException
+  {
+    String intro = (String) getAttribute("qrcodeintro");
+    if (null == intro)
+    {
+      return "Bequem bezahlen mit Girocode. Einfach mit der Banking-App auf dem Handy abscannen.";
+    }
+    return intro;
+  }
+
+  @Override
+  public void setQRCodeIntro(String intro) throws RemoteException
+  {
+    setAttribute("qrcodeintro", intro);
+  }
+  
+  @Override 
+  public Boolean getUnterschriftdrucken() throws RemoteException
+  {
+    return Util.getBoolean(getAttribute("unterschriftdrucken"));
+  }
+
+  @Override
+  public void setUnterschriftdrucken(Boolean unterschriftdrucken) throws RemoteException
+  {
+    setAttribute("unterschriftdrucken", unterschriftdrucken);
+  }
+  
+  @Override
+  public byte[] getUnterschrift() throws RemoteException
+  {
+    return (byte[]) getAttribute("unterschrift");
+  }
+
+  @Override
+  public void setUnterschrift(byte[] unterschrift) throws RemoteException
+  {
+    setAttribute("unterschrift", unterschrift);
+  }
+  
+  @Override
+  public Boolean getAnhangSpeichern() throws RemoteException
+  {
+    return Util.getBoolean(getAttribute("anhangspeichern"));
+  }
+
+  @Override
+  public void setAnhangSpeichern(Boolean anhangspeichern) throws RemoteException
+  {
+    setAttribute("anhangspeichern", anhangspeichern);
+  }
+  
+  @Override
+  public Boolean getBuchungsklasseInBuchung() throws RemoteException
+  {
+    return Util.getBoolean(getAttribute("bkinbuchung"));
+  }
+
+  @Override
+  public void setBuchungsklasseInBuchung(Boolean bkinbuchung) throws RemoteException
+  {
+    setAttribute("bkinbuchung", bkinbuchung);
+  }
+
+  public Boolean getSummenAnlagenkonto() throws RemoteException
+  {
+    return Util.getBoolean(getAttribute("summenanlagenkonto"));
+  }
+
+  @Override
+  public void setSummenAnlagenkonto(Boolean summenanlagenkonto) throws RemoteException
+  {
+    setAttribute("summenanlagenkonto", summenanlagenkonto);
+  }
+  
+  @Override
+  public Double getAfaRestwert() throws RemoteException
+  {
+    return (Double) getAttribute("afarestwert");
+  }
+
+  @Override
+  public void setAfaRestwert(Double afarestwert) throws RemoteException
+  {
+    setAttribute("afarestwert", afarestwert);
+  }
+  
+  @Override
+  public Boolean getAfaInJahresabschluss() throws RemoteException
+  {
+    return Util.getBoolean(getAttribute("afainjahresabschluss"));
+  }
+
+  @Override
+  public void setAfaInJahresabschluss(Boolean afainjahresabschluss)
+      throws RemoteException
+  {
+    setAttribute("afainjahresabschluss", afainjahresabschluss);
+  }
+
+  @Override
+  public String getBeitragAltersstufen() throws RemoteException
+  {
+    String altersstufe = (String) getAttribute("beitragaltersstufen");
+    if (altersstufe == null)
+    {
+      return "0-99";
+    }
+    return altersstufe;
+  }
+
+  @Override
+  public void setBeitragAltersstufen(String altersstufen) throws RemoteException
+  {
+    setAttribute("beitragaltersstufen", altersstufen);
+  }
 }
