@@ -25,7 +25,6 @@ import java.util.HashMap;
 import java.util.Map;
 
 import de.jost_net.JVerein.Einstellungen;
-import de.jost_net.JVerein.io.BeitragsUtil;
 import de.jost_net.JVerein.io.Adressbuch.Adressaufbereitung;
 import de.jost_net.JVerein.keys.ArtBeitragsart;
 import de.jost_net.JVerein.keys.Datentyp;
@@ -36,8 +35,8 @@ import de.jost_net.JVerein.keys.Zahlungsweg;
 import de.jost_net.JVerein.rmi.Adresstyp;
 import de.jost_net.JVerein.rmi.Beitragsgruppe;
 import de.jost_net.JVerein.rmi.Felddefinition;
-import de.jost_net.JVerein.rmi.Lastschrift;
 import de.jost_net.JVerein.rmi.Mitglied;
+import de.jost_net.JVerein.rmi.MitgliedDokument;
 import de.jost_net.JVerein.rmi.Mitgliedfoto;
 import de.jost_net.JVerein.rmi.Zusatzfelder;
 import de.jost_net.JVerein.util.Datum;
@@ -46,11 +45,12 @@ import de.jost_net.JVerein.util.JVDateFormatTTMMJJJJ;
 import de.jost_net.OBanToo.SEPA.BIC;
 import de.jost_net.OBanToo.SEPA.IBAN;
 import de.jost_net.OBanToo.SEPA.SEPAException;
-import de.jost_net.OBanToo.SEPA.SEPAException.Fehler;
-import de.jost_net.OBanToo.SEPA.Basislastschrift.MandatSequence;
 import de.willuhn.datasource.db.AbstractDBObject;
 import de.willuhn.datasource.rmi.DBIterator;
+import de.willuhn.datasource.rmi.DBService;
 import de.willuhn.datasource.rmi.ResultSetExtractor;
+import de.willuhn.jameica.messaging.QueryMessage;
+import de.willuhn.jameica.system.Application;
 import de.willuhn.logging.Logger;
 import de.willuhn.util.ApplicationException;
 
@@ -80,9 +80,26 @@ public class MitgliedImpl extends AbstractDBObject implements Mitglied
   }
 
   @Override
-  protected void deleteCheck()
+  protected void deleteCheck() throws ApplicationException
   {
-    //
+    try
+    {
+      // Falls das Mitglied fr andere zahlt kann man nicht lschen
+      DBIterator<Mitglied> famang = Einstellungen.getDBService()
+          .createList(Mitglied.class);
+      famang.addFilter("zahlerid = " + getID());
+      if (famang.hasNext())
+      {
+        throw new ApplicationException(
+            "Dieses Mitglied zahlt noch fr andere Mitglieder. Zunchst Beitragsart der Angehrigen ndern!");
+      }
+    }
+    catch (RemoteException e)
+    {
+      String fehler = "Mitglied kann nicht gelscht werden. Siehe system log";
+      Logger.error(fehler, e);
+      throw new ApplicationException(fehler);
+    }
   }
 
   @Override
@@ -106,26 +123,26 @@ public class MitgliedImpl extends AbstractDBObject implements Mitglied
     checkExterneMitgliedsnummer();
 
     if (getPersonenart() == null
-        || (!getPersonenart().equals("n") && !getPersonenart().equals("j")))
+        || (!getPersonenart().equalsIgnoreCase("n") && !getPersonenart().equalsIgnoreCase("j")))
     {
-      throw new ApplicationException("Personenstatus ist nicht 'n' oder 'j'");
+      throw new ApplicationException("Personenstatus ist nicht 'N' oder 'J'");
     }
     if (getName() == null || getName().length() == 0)
     {
       throw new ApplicationException("Bitte Namen eingeben");
     }
-    if (getPersonenart().equals("n")
+    if (getPersonenart().equalsIgnoreCase("n")
         && (getVorname() == null || getVorname().length() == 0))
     {
       throw new ApplicationException("Bitte Vornamen eingeben");
     }
-    if (getAdresstyp().getJVereinid() == 1 && getPersonenart().equals("n")
+    if (getAdresstyp().getJVereinid() == 1 && getPersonenart().equalsIgnoreCase("n")
         && getGeburtsdatum().getTime() == Einstellungen.NODATE.getTime()
         && Einstellungen.getEinstellung().getGeburtsdatumPflicht())
     {
       throw new ApplicationException("Bitte Geburtsdatum eingeben");
     }
-    if (getAdresstyp().getJVereinid() == 1 && getPersonenart().equals("n")
+    if (getAdresstyp().getJVereinid() == 1 && getPersonenart().equalsIgnoreCase("n")
         && Einstellungen.getEinstellung().getGeburtsdatumPflicht())
     {
       Calendar cal1 = Calendar.getInstance();
@@ -146,7 +163,7 @@ public class MitgliedImpl extends AbstractDBObject implements Mitglied
             "Ist das Mitglied wirklich lter als 150 Jahre?");
       }
     }
-    if (getPersonenart().equals("n") && getGeschlecht() == null)
+    if (getPersonenart().equalsIgnoreCase("n") && getGeschlecht() == null)
     {
       throw new ApplicationException("Bitte Geschlecht auswhlen");
     }
@@ -164,18 +181,15 @@ public class MitgliedImpl extends AbstractDBObject implements Mitglied
     {
       throw new ApplicationException("Bitte Eintrittsdatum eingeben");
     }
-    if (getAdresstyp().getJVereinid() == 1
-        && getZahlungsweg() == Zahlungsweg.BASISLASTSCHRIFT
-        && BeitragsUtil.getBeitrag(
-            Einstellungen.getEinstellung().getBeitragsmodel(),
-            this.getZahlungstermin(), this.getZahlungsrhythmus().getKey(),
-            this.getBeitragsgruppe(), new Date(), getEintritt(),
-            getAustritt()) > 0)
+    if (getZahlungsweg() == Zahlungsweg.BASISLASTSCHRIFT)
     {
-      if (getBic() == null || getBic().length() == 0 || getIban() == null
-          || getIban().length() == 0)
+      if (getIban() == null || getIban().length() == 0)
       {
-        throw new ApplicationException("Bitte BIC und IBAN eingeben");
+        throw new ApplicationException("Bitte IBAN eingeben");
+      }
+      if (getMandatDatum() == Einstellungen.NODATE)
+      {
+        throw new ApplicationException("Bitte Datum des Mandat eingeben");
       }
     }
     if (getIban() != null && getIban().length() != 0)
@@ -186,10 +200,7 @@ public class MitgliedImpl extends AbstractDBObject implements Mitglied
       }
       catch (SEPAException e)
       {
-        if (e.getFehler() != Fehler.UNGUELTIGES_LAND)
-        {
-          throw new ApplicationException(e.getMessage());
-        }
+        throw new ApplicationException("Ungltige IBAN");
       }
     }
     if (getBic() != null && getBic().length() != 0)
@@ -200,10 +211,7 @@ public class MitgliedImpl extends AbstractDBObject implements Mitglied
       }
       catch (SEPAException e)
       {
-        if (!e.getMessage().startsWith("Ungltiges Land"))
-        {
-          throw new ApplicationException(e.getMessage());
-        }
+        throw new ApplicationException("Ungltige BIC");
       }
     }
     if (getZahlungsrhythmus() == null)
@@ -216,30 +224,72 @@ public class MitgliedImpl extends AbstractDBObject implements Mitglied
       throw new ApplicationException(
           "Bei verstorbenem Mitglied muss das Austrittsdatum gefllt sein!");
     }
-    if (getAustritt() != null || getKuendigung() != null)
+    if (getAustritt() != null)
     {
       // Person ist ausgetreten
-      // Hat das Mitglied fr andere gezahlt?
-      if (getBeitragsgruppe().getBeitragsArt() == ArtBeitragsart.FAMILIE_ZAHLER)
+      // Ist das Mitglied Vollzahler in einem Familienverband?
+      if (getBeitragsgruppe() != null && getBeitragsgruppe().getBeitragsArt() != ArtBeitragsart.FAMILIE_ANGEHOERIGER)
       {
-        // ja
         DBIterator<Mitglied> famang = Einstellungen.getDBService()
             .createList(Mitglied.class);
         famang.addFilter("zahlerid = " + getID());
-        famang.addFilter("austritt is null");
+        famang.addFilter("(austritt is null or austritt > ?)",getAustritt());
         if (famang.hasNext())
         {
           throw new ApplicationException(
-              "Dieses Mitglied zahlt noch fr andere Mitglieder. Zunchst Beitragsart der Angehrigen ndern!");
+              "Dieses Mitglied ist Vollzahler fr andere. Zunchst Beitragsart der Angehrigen ndern!");
         }
       }
     }
+    // Ist das Mitglied Teil eines Familienverbandes?
+    if (getBeitragsgruppe() != null && getBeitragsgruppe().getBeitragsArt() == ArtBeitragsart.FAMILIE_ANGEHOERIGER
+        && getZahlerID() != null)
+    {
+      // ja, suche Vollzahler. Er darf nicht, bzw nicht frher, ausgetreten sein!
+      DBIterator<Mitglied> zahler = Einstellungen.getDBService()
+          .createList(Mitglied.class);
+      zahler.addFilter("id = " + getZahlerID());
+      if(getAustritt() != null)
+        zahler.addFilter("(austritt is not null and austritt < ?)",getAustritt());
+      Mitglied z = null;
+      if(zahler.hasNext())
+        z = zahler.next();
+      if (z != null && ((Mitglied) z).getAustritt() != null)
+      {
+        throw new ApplicationException(
+            "Der ausgewhlte Vollzahler ist ausgetreten zu " + z.getAustritt() + ". Bitte anderen Vollzahler whlen!");
+      }
+      if(z != null && ((Mitglied)z).getEintritt().after(new Date()) && ((Mitglied)z).getEintritt().after(getEintritt()))
+      {
+    	throw new ApplicationException(
+    	    "Der ausgewhlte Vollzahler tritt erst ein zu " + z.getEintritt() + ". Bitte anderen Vollzahler whlen!");
+      }
+    }
+    // Check ob das Mitglied vorher ein Vollzahler eines Familienverbandes war
+    if (getBeitragsgruppe() != null && getBeitragsgruppe().getBeitragsArt() == ArtBeitragsart.FAMILIE_ANGEHOERIGER)
+    {
+      // Es darf keine Familienangehrigen geben
+      DBIterator<Mitglied> famang = Einstellungen.getDBService()
+          .createList(Mitglied.class);
+      famang.addFilter("zahlerid = " + getID());
+      if (famang.hasNext())
+      {
+        throw new ApplicationException(
+            "Dieses Mitglied ist Vollzahler in einem Familienverband.. Zunchst Beitragsart der Angehrigen ndern!");
+      }
+    }
     if (getBeitragsgruppe() != null
         && getBeitragsgruppe()
             .getBeitragsArt() == ArtBeitragsart.FAMILIE_ANGEHOERIGER
         && getZahlerID() == null)
     {
-      throw new ApplicationException("Bitte Zahler auswhlen!");
+      throw new ApplicationException("Bitte Vollzahler auswhlen!");
+    }
+    
+    // Individueller Beitrag darf nicht kleiner als 0 sein
+    if (getIndividuellerBeitrag() != null && getIndividuellerBeitrag() < 0)
+    {
+      throw new ApplicationException("Individueller Beitrag darf nicht negativ sein!");
     }
   }
 
@@ -259,7 +309,7 @@ public class MitgliedImpl extends AbstractDBObject implements Mitglied
     if (Einstellungen.getEinstellung().getExterneMitgliedsnummer() == false)
       return;
 
-    if (getExterneMitgliedsnummer() == null)
+    if (getExterneMitgliedsnummer() == null || getExterneMitgliedsnummer().isEmpty())
     {
       throw new ApplicationException("Externe Mitgliedsnummer fehlt");
     }
@@ -297,10 +347,6 @@ public class MitgliedImpl extends AbstractDBObject implements Mitglied
   @Override
   protected Class<?> getForeignObject(String field)
   {
-    if ("beitragsgruppe".equals(field))
-    {
-      return Beitragsgruppe.class;
-    }
     if ("foto".equals(field))
     {
       return Mitgliedfoto.class;
@@ -574,7 +620,7 @@ public class MitgliedImpl extends AbstractDBObject implements Mitglied
     Integer vers = (Integer) getAttribute("mandatversion");
     if (vers == null)
     {
-      vers = new Integer(0);
+      vers = Integer.valueOf(0);
     }
     return vers;
   }
@@ -585,29 +631,6 @@ public class MitgliedImpl extends AbstractDBObject implements Mitglied
     setAttribute("mandatversion", mandatversion);
   }
 
-  @Override
-  public MandatSequence getMandatSequence() throws RemoteException
-  {
-    DBIterator<Lastschrift> it = Einstellungen.getDBService()
-        .createList(Lastschrift.class);
-    it.addFilter("mandatid = ?", getMandatID());
-    if (it.size() == 0)
-    {
-      return MandatSequence.FRST;
-    }
-    else
-    {
-      return MandatSequence.RCUR;
-    }
-  }
-
-  @Override
-  public void setMandatSequence(MandatSequence mandatsequence)
-      throws RemoteException
-  {
-    setAttribute("mandatsequence", mandatsequence.getTxt());
-  }
-
   @Override
   public String getMandatID() throws RemoteException
   {
@@ -1005,7 +1028,12 @@ public class MitgliedImpl extends AbstractDBObject implements Mitglied
   @Override
   public Beitragsgruppe getBeitragsgruppe() throws RemoteException
   {
-    return (Beitragsgruppe) getAttribute("beitragsgruppe");
+    Object o = (Object) super.getAttribute("beitragsgruppe");
+    if (o == null)
+      return null;
+   
+    Cache cache = Cache.get(Beitragsgruppe.class,true);
+    return (Beitragsgruppe) cache.get(o);
   }
 
   @Override
@@ -1023,12 +1051,7 @@ public class MitgliedImpl extends AbstractDBObject implements Mitglied
   @Override
   public Double getIndividuellerBeitrag() throws RemoteException
   {
-    Double d = (Double) getAttribute("individuellerbeitrag");
-    if (d == null)
-    {
-      return new Double(0);
-    }
-    return d;
+    return (Double) getAttribute("individuellerbeitrag");
   }
 
   @Override
@@ -1176,7 +1199,7 @@ public class MitgliedImpl extends AbstractDBObject implements Mitglied
   {
     if (fieldName.equals("idint"))
     {
-      return new Integer(getID());
+      return Integer.valueOf(getID());
     }
     if (fieldName.equals("namevorname"))
     {
@@ -1192,6 +1215,13 @@ public class MitgliedImpl extends AbstractDBObject implements Mitglied
     }
     else if (fieldName.startsWith("zusatzfelder_"))
     {
+      Long l = (Long) super.getAttribute("beitragsgruppe");
+      if (l == null)
+        return null;
+       
+      Cache cache = Cache.get(Beitragsgruppe.class,true);
+      cache.get(l);
+        
       DBIterator<Felddefinition> it = Einstellungen.getDBService()
           .createList(Felddefinition.class);
       it.addFilter("name = ?", new Object[] { fieldName.substring(13) });
@@ -1231,6 +1261,8 @@ public class MitgliedImpl extends AbstractDBObject implements Mitglied
     {
       return getAlter();
     }
+    else if("beitragsgruppe".equals(fieldName))
+    	return getBeitragsgruppe();
     return super.getAttribute(fieldName);
   }
 
@@ -1261,4 +1293,20 @@ public class MitgliedImpl extends AbstractDBObject implements Mitglied
     return d;
   }
 
+  @Override
+  public void delete() throws RemoteException, ApplicationException
+  {
+    DBService service = Einstellungen.getDBService();
+    DBIterator<MitgliedDokument> docs = service.createList(MitgliedDokument.class);
+    docs.addFilter("referenz = ?", new Object[] { this.getID() });
+    while (docs.hasNext())
+    {
+      QueryMessage qm = new QueryMessage(
+          ((MitgliedDokument) docs.next()).getUUID(), null);
+      Application.getMessagingFactory().getMessagingQueue(
+          "jameica.messaging.del").sendSyncMessage(qm);
+    }
+    super.delete();
+  }
+  
 }
