@@ -235,7 +235,7 @@ public class SpendenbescheinigungImpl extends AbstractDBObject
     Double ret = (Double) getAttribute("betrag");
     if (ret == null)
     {
-      ret = new Double(0);
+      ret = Double.valueOf(0);
     }
     return ret;
   }
@@ -350,6 +350,18 @@ public class SpendenbescheinigungImpl extends AbstractDBObject
   @Override
   public Object getAttribute(String fieldName) throws RemoteException
   {
+    if ("id-int".equals(fieldName))
+    {
+      try
+      {
+        return Integer.valueOf(getID());
+      }
+      catch (Exception e)
+      {
+        Logger.error("unable to parse id: " + getID());
+        return getID();
+      }
+    }
     return super.getAttribute(fieldName);
   }
 
@@ -357,26 +369,15 @@ public class SpendenbescheinigungImpl extends AbstractDBObject
   public void store() throws RemoteException, ApplicationException
   {
     super.store();
-    Long id = new Long(getID());
-    for (Buchung b : buchungen)
+    Long id = Long.valueOf(getID());
+    if (buchungen != null)
     {
-      b.setSpendenbescheinigungId(id);
-      b.store();
-    }
-  }
-
-  @Override
-  public void delete() throws RemoteException, ApplicationException
-  {
-    if (getSpendenart() == Spendenart.GELDSPENDE)
-    {
-      for (Buchung b : getBuchungen())
+      for (Buchung b : buchungen)
       {
-        b.setSpendenbescheinigungId(null);
+        b.setSpendenbescheinigungId(id);
         b.store();
       }
     }
-    super.delete();
   }
 
   @Override
@@ -402,7 +403,7 @@ public class SpendenbescheinigungImpl extends AbstractDBObject
   {
     if (mitglied != null)
     {
-      setAttribute("mitglied", new Integer(mitglied.getID()));
+      setAttribute("mitglied", Integer.valueOf(mitglied.getID()));
     }
     else
     {
@@ -572,7 +573,7 @@ public class SpendenbescheinigungImpl extends AbstractDBObject
       this.setSpendedatum(new Date());
       this.setSpendenart(Spendenart.GELDSPENDE);
       this.setUnterlagenWertermittlung(true);
-      this.setZeile1("Herrn");
+      this.setZeile1("Herr");
       this.setZeile2("Dr. Willi Wichtig");
       this.setZeile3("Hinterm Bahnhof 1");
       this.setZeile4("12345 Testenhausen");
@@ -584,6 +585,15 @@ public class SpendenbescheinigungImpl extends AbstractDBObject
         + getZeile3() + newLineStr + getZeile4() + newLineStr + getZeile5()
         + newLineStr + getZeile6() + newLineStr + getZeile7() + newLineStr;
     map.put(SpendenbescheinigungVar.EMPFAENGER.getName(), empfaenger);
+    String anrede = (getZeile1().length() > 0) ? getZeile1() + " " + getZeile2() : getZeile2();
+    map.put(SpendenbescheinigungVar.ANREDE.getName(), anrede);
+    map.put(SpendenbescheinigungVar.ZEILE1.getName(), getZeile1());
+    map.put(SpendenbescheinigungVar.ZEILE2.getName(), getZeile2());
+    map.put(SpendenbescheinigungVar.ZEILE3.getName(), getZeile3());
+    map.put(SpendenbescheinigungVar.ZEILE4.getName(), getZeile4());
+    map.put(SpendenbescheinigungVar.ZEILE5.getName(), getZeile5());
+    map.put(SpendenbescheinigungVar.ZEILE6.getName(), getZeile6());
+    map.put(SpendenbescheinigungVar.ZEILE7.getName(), getZeile7());
     Double dWert = getBetrag();
     // Hier keinen String, sondern ein Double-Objekt in die Map stellen,
     // damit eine rechtsbndige Ausrichtung des Betrages in der Formular-
@@ -596,7 +606,7 @@ public class SpendenbescheinigungImpl extends AbstractDBObject
     {
       String betraginworten = GermanNumber.toString(dWert.longValue());
       map.put(SpendenbescheinigungVar.BETRAGINWORTEN.getName(),
-          "*" + betraginworten + "*");
+          "-" + betraginworten + "-");
     }
     catch (Exception e)
     {
@@ -679,6 +689,10 @@ public class SpendenbescheinigungImpl extends AbstractDBObject
       map.put(SpendenbescheinigungVar.SPENDENZEITRAUM.getName(),
           String.format("%s bis %s", spendedatum, zeitraumende));
       StringBuilder bl = new StringBuilder();
+      StringBuilder bl_daten = new StringBuilder();
+      StringBuilder bl_art = new StringBuilder();
+      StringBuilder bl_verzicht = new StringBuilder();
+      StringBuilder bl_betrag = new StringBuilder();
       if (gc.get(GregorianCalendar.YEAR) <= 2012)
       {
         bl.append(StringTool.rpad("Datum", 10));
@@ -782,11 +796,15 @@ public class SpendenbescheinigungImpl extends AbstractDBObject
         {
           bl.append(StringTool.rpad(
               new JVDateFormatTTMMJJJJ().format(b.getDatum()), colDatumLen));
+          bl_daten.append(new JVDateFormatTTMMJJJJ().format(b.getDatum()));
+          bl_daten.append(newLineStr);
           bl.append("  ");
           if (printBuchungsart)
           {
             bl.append(StringTool.rpad(b.getBuchungsart().getBezeichnung(),
                 colArtLen));
+            bl_art.append(b.getBuchungsart().getBezeichnung());
+            bl_art.append(newLineStr);
           }
           else
           {
@@ -797,16 +815,21 @@ public class SpendenbescheinigungImpl extends AbstractDBObject
           {
             bl.append(StringTool.rpad(
                 StringTool.lpad("ja", colVerzichtLen / 2 - 2), colVerzichtLen));
+            bl_verzicht.append("ja");
           }
           else
           {
             bl.append(
                 StringTool.rpad(StringTool.lpad("nein", colVerzichtLen / 2 - 2),
                     colVerzichtLen));
+            bl_verzicht.append("nein");
           }
+          bl_verzicht.append(newLineStr);
           bl.append("  ");
           String str = Einstellungen.DECIMALFORMAT.format(b.getBetrag());
           bl.append(StringTool.lpad(str, colBetragLen));
+          bl_betrag.append(StringTool.lpad(str, colBetragLen));
+          bl_betrag.append(newLineStr);
           bl.append(newLineStr);
         }
 
@@ -832,6 +855,10 @@ public class SpendenbescheinigungImpl extends AbstractDBObject
         bl.append(newLineStr);
       }
       map.put(SpendenbescheinigungVar.BUCHUNGSLISTE.getName(), bl.toString());
+      map.put(SpendenbescheinigungVar.BUCHUNGSLISTE_DATEN.getName(), bl_daten.toString());
+      map.put(SpendenbescheinigungVar.BUCHUNGSLISTE_ART.getName(), bl_art.toString());
+      map.put(SpendenbescheinigungVar.BUCHUNGSLISTE_VERZICHT.getName(), bl_verzicht.toString());
+      map.put(SpendenbescheinigungVar.BUCHUNGSLISTE_BETRAG.getName(), bl_betrag.toString());
     }
     else
     {
